@@ -1,5 +1,5 @@
 SANE_Status
-sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
+sane_init (SANE_Int * version_code, SANE_Auth_Callback UNUSEDARG authorize)
 {
   char devnam[PATH_MAX] = "/dev/scanner";
   FILE *fp;
@@ -97,7 +97,7 @@ sane_exit (void)
       next = dev->next;
       free ((void *) dev->sane.name);
       free ((void *) dev->sane.model);
-      free (dev);
+      free ((void *) dev);
     }
 
   DBG (1, "<< sane_exit\n");
@@ -106,7 +106,8 @@ sane_exit (void)
 /**************************************************************************/
 
 SANE_Status
-sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
+sane_get_devices (const SANE_Device *** device_list,
+SANE_Bool UNUSEDARG local_only)
 {
   static const SANE_Device **devlist = 0;
   CANON_Device *dev;
@@ -370,8 +371,6 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	  *(SANE_Word *) val = s->val[option].w;
 	  DBG (21, "value for option %s: %d\n", option_name[option],
 	       s->val[option].w);
-	  if (info)
-	    *info |= SANE_INFO_RELOAD_PARAMS;
 	  return (SANE_STATUS_GOOD);
 
 	case OPT_GAMMA_VECTOR:
@@ -434,33 +433,10 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	case OPT_TPU_DCM:
 	case OPT_TPU_FILMTYPE:
 	case OPT_MODE:
-	  strcpy (val, s->val[option].s);
-	  if (info)
-	    *info |= SANE_INFO_RELOAD_PARAMS;
-	  DBG (21, "value for option %s: %s\n", option_name[option],
-	       s->val[option].s);
-	  return (SANE_STATUS_GOOD);
-
 	case OPT_NEGATIVE:
-	  strcpy (val, s->val[option].s);
-	  if (info)
-	    *info |= SANE_INFO_RELOAD_PARAMS | SANE_INFO_RELOAD_OPTIONS;
-	  DBG (21, "value for option %s: %s\n", option_name[option],
-	       s->val[option].s);
-	  return (SANE_STATUS_GOOD);
-
 	case OPT_NEGATIVE_TYPE:
-	  strcpy (val, s->val[option].s);
-	  if (info)
-	    *info |= SANE_INFO_RELOAD_PARAMS | SANE_INFO_RELOAD_OPTIONS;
-	  DBG (21, "value for option %s: %s\n", option_name[option],
-	       s->val[option].s);
-	  return (SANE_STATUS_GOOD);
-
 	case OPT_SCANNING_SPEED:
 	  strcpy (val, s->val[option].s);
-	  if (info)
-	    *info |= SANE_INFO_RELOAD_PARAMS | SANE_INFO_RELOAD_OPTIONS;
 	  DBG (21, "value for option %s: %s\n", option_name[option],
 	       s->val[option].s);
 	  return (SANE_STATUS_GOOD);
@@ -657,20 +633,19 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 
 	case OPT_FLATBED_ONLY:
 	  s->val[option].w = *(SANE_Word *) val;
-	  if (s->hw->adf.Status != ADF_STAT_NONE && s->val[option].w )
+	  if (s->hw->adf.Status != ADF_STAT_NONE && s->val[option].w)
 	    {					/* switch on */
 	      s->hw->adf.Priority |= 0x03;	/* flatbed mode */
 	      s->hw->adf.Feeder &= 0x00;	/* autofeed off (default) */
 	      s->hw->adf.Status = ADF_STAT_DISABLED;
-	      s->val[option].w = SANE_TRUE;
 	    }		/* if it isn't connected, don't bother fixing */
 	  return SANE_STATUS_GOOD;
 
 	case OPT_TPU_ON:
-	  if (s->val[OPT_TPU_ON].w == TPU_STAT_INACTIVE)	/* switch on */
+	  s->val[option].w = *(SANE_Word *) val;
+	  if (s->val[option].w)			/* switch on */
 	    {
-	      s->val[OPT_TPU_ON].w = TPU_STAT_ACTIVE;
-	      s->opt[OPT_TPU_ON].title = "Turn Off the Transparency Unit";
+	      s->hw->tpu.Status = TPU_STAT_ACTIVE;
 	      s->opt[OPT_TPU_TRANSPARENCY].cap &=
 		(s->hw->tpu.ControlMode == 3) ? ~SANE_CAP_INACTIVE : ~0;
 	      s->opt[OPT_TPU_FILMTYPE].cap &=
@@ -678,8 +653,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	    }
 	  else			/* switch off */
 	    {
-	      s->val[OPT_TPU_ON].w = TPU_STAT_INACTIVE;
-	      s->opt[OPT_TPU_ON].title = "Turn On Transparency Unit";
+	      s->hw->tpu.Status = TPU_STAT_INACTIVE;
 	      s->opt[OPT_TPU_TRANSPARENCY].cap |= SANE_CAP_INACTIVE;
 	      s->opt[OPT_TPU_FILMTYPE].cap |= SANE_CAP_INACTIVE;
 	    }
@@ -697,13 +671,13 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	  s->opt[OPT_TPU_TRANSPARENCY].cap |= SANE_CAP_INACTIVE;
 	  s->opt[OPT_TPU_FILMTYPE].cap |= SANE_CAP_INACTIVE;
 	  if (!strcmp (s->val[OPT_TPU_DCM].s,
-		       "Correction according to Transparency Ratio"))
+	  SANE_I18N("Correction according to transparency ratio")))
 	    {
 	      s->hw->tpu.ControlMode = 3;
 	      s->opt[OPT_TPU_TRANSPARENCY].cap &= ~SANE_CAP_INACTIVE;
 	    }
 	  else if (!strcmp (s->val[OPT_TPU_DCM].s,
-			    "Correction according to Film type"))
+	  SANE_I18N("Correction according to film type")))
 	    {
 	      s->hw->tpu.ControlMode = 1;
 	      s->opt[OPT_TPU_FILMTYPE].cap &= ~SANE_CAP_INACTIVE;
@@ -726,7 +700,8 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	  if (s->val[option].s)
 	    free (s->val[option].s);
 	  s->val[option].s = strdup (val);
-	  if (!strcmp (val, "Lineart") || !strcmp (val, "Halftone"))
+	  if (!strcmp (val, SANE_I18N("Lineart"))
+	  || !strcmp (val, SANE_I18N("Halftone")))
 	    {
 	      /* For Lineart and Halftone: */
 	      /* Enable "threshold" */
@@ -753,7 +728,8 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	      s->opt[OPT_CUSTOM_GAMMA].cap &= ~SANE_CAP_INACTIVE;
 	      if (s->val[OPT_CUSTOM_GAMMA].w)
 		{
-		  if (!strcmp (val, "Color") || !strcmp (val, "Fine color"))
+		  if (!strcmp (val, SANE_I18N("Color"))
+		  || !strcmp (val, SANE_I18N("Fine color")))
 		    {
 		      s->opt[OPT_CUSTOM_GAMMA_BIND].cap &= ~SANE_CAP_INACTIVE;
 		      if (s->val[OPT_CUSTOM_GAMMA_BIND].w == SANE_TRUE)
@@ -797,7 +773,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	  if (s->val[option].s)
 	    free (s->val[option].s);
 	  s->val[option].s = strdup (val);
-	  if (!strcmp (val, "Negatives"))
+	  if (!strcmp (val, SANE_I18N("Negatives")))
 	    {
 	      s->RIF = 0;
 	      s->opt[OPT_NEGATIVE_TYPE].cap &= ~SANE_CAP_INACTIVE;
@@ -958,10 +934,10 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	    {
 	      const char *mode = s->val[OPT_MODE].s;
 
-	      if (!strcmp (mode, "Gray"))
+	      if (!strcmp (mode, SANE_I18N("Gray")))
 		s->opt[OPT_GAMMA_VECTOR].cap &= ~SANE_CAP_INACTIVE;
-	      else if (!strcmp (mode, "Color")
-		       || !strcmp (mode, "Fine color"))
+	      else if (!strcmp (mode, SANE_I18N("Color"))
+	      || !strcmp (mode, SANE_I18N("Fine color")))
 		{
 		  s->opt[OPT_CUSTOM_GAMMA_BIND].cap &= ~SANE_CAP_INACTIVE;
 		  if (s->val[OPT_CUSTOM_GAMMA_BIND].w)
@@ -1081,7 +1057,8 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters *params)
 	}
 
       mode = s->val[OPT_MODE].s;
-      if (!strcmp (mode, "Lineart") || !strcmp (mode, "Halftone"))
+      if (!strcmp (mode, SANE_I18N("Lineart"))
+      || !strcmp (mode, SANE_I18N("Halftone")))
 	{
 	  s->params.format = SANE_FRAME_GRAY;
 	  s->params.bytes_per_line = s->params.pixels_per_line / 8;
@@ -1089,13 +1066,14 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters *params)
 	  s->params.pixels_per_line = s->params.bytes_per_line * 8;
 	  s->params.depth = 1;
 	}
-      else if (!strcmp (mode, "Gray"))
+      else if (!strcmp (mode, SANE_I18N("Gray")))
 	{
 	  s->params.format = SANE_FRAME_GRAY;
 	  s->params.bytes_per_line = s->params.pixels_per_line;
 	  s->params.depth = 8;
 	}
-      else if (!strcmp (mode, "Color") || !strcmp (mode, "Fine color"))
+      else if (!strcmp (mode, SANE_I18N("Color"))
+      || !strcmp (mode, SANE_I18N("Fine color")))
 	{
 	  s->params.format = SANE_FRAME_RGB;
 	  s->params.bytes_per_line = 3 * s->params.pixels_per_line;
@@ -1179,7 +1157,7 @@ sane_start (SANE_Handle handle)
 
   s->scanning = SANE_FALSE;
 
-  if ((s->hw->adf.Status == SANE_TRUE)
+  if ((s->hw->adf.Status != ADF_STAT_NONE)
       && (s->val[OPT_FLATBED_ONLY].w != SANE_TRUE)
       && (s->hw->adf.Problem != 0))
     {
@@ -1201,7 +1179,7 @@ sane_start (SANE_Handle handle)
 	}
       return status;
     }
-  else if ((s->hw->adf.Status == SANE_TRUE)
+  else if ((s->hw->adf.Status != ADF_STAT_NONE)
 	   && (s->val[OPT_FLATBED_ONLY].w == SANE_TRUE))
     {
       set_adf_mode (s->fd, s->hw->adf.Priority);
@@ -1249,7 +1227,7 @@ sane_start (SANE_Handle handle)
       return (SANE_STATUS_INVAL);
     }
   for (i = 0; i < buf_size; i++)
-    DBG (3, "scan mode control byte[%d] = %d\n", i, ebuf[i]);
+    DBG (3, "scan mode control byte[%d] = %d\n", (int) i, ebuf[i]);
 
   if (s->hw->adf.Status != ADF_STAT_NONE)
     {
@@ -1265,7 +1243,8 @@ sane_start (SANE_Handle handle)
 	  return (SANE_STATUS_INVAL);
 	}
       for (i = 0; i < buf_size; i++)
-	DBG (3, "scan mode control byte[%d] = %d\n", i, ebuf[i]);
+	DBG (3, "scan mode control byte[%d] = %d\n", (int) i,
+	ebuf[i]);
     }
 
   mode_str = s->val[OPT_MODE].s;
@@ -1287,7 +1266,8 @@ sane_start (SANE_Handle handle)
 
   if (s->hw->info.model != CS2700 && s->hw->info.model != FS2710)
     {
-      if (!strcmp (mode_str, "Lineart") || !strcmp (mode_str, "Halftone"))
+      if (!strcmp (mode_str, SANE_I18N("Lineart"))
+      || !strcmp (mode_str, SANE_I18N("Halftone")))
 	s->RIF = s->val[OPT_HNEGATIVE].w;
       else
 	s->RIF = !s->val[OPT_HNEGATIVE].w;
@@ -1319,27 +1299,28 @@ sane_start (SANE_Handle handle)
       s->ShadowB = s->val[OPT_SHADOW_B].w;
     }
 
-  if (!strcmp (mode_str, "Lineart"))
+  if (!strcmp (mode_str, SANE_I18N("Lineart")))
     {
       mode = 4;
       s->image_composition = 0;
     }
-  else if (!strcmp (mode_str, "Halftone"))
+  else if (!strcmp (mode_str, SANE_I18N("Halftone")))
     {
       mode = 4;
       s->image_composition = 1;
     }
-  else if (!strcmp (mode_str, "Gray"))
+  else if (!strcmp (mode_str, SANE_I18N("Gray")))
     {
       mode = 5;
       s->image_composition = 2;
     }
-  else if (!strcmp (mode_str, "Color") || !strcmp (mode_str, "Fine color"))
+  else if (!strcmp (mode_str, SANE_I18N("Color"))
+  || !strcmp (mode_str, SANE_I18N("Fine color")))
     {
       mode = 6;
       s->image_composition = 5;
     }
-  else if (!strcmp (mode_str, "Raw"))
+  else if (!strcmp (mode_str, SANE_I18N("Raw")))
     {
       mode = 6;
       s->image_composition = 5;
@@ -1414,7 +1395,8 @@ sane_start (SANE_Handle handle)
 #if 0
       wbuf[34] = (((600 < s->val[OPT_X_RESOLUTION].w)
 	|| (600 < s->val[OPT_Y_RESOLUTION].w))
-	&& (strcmp (s->val[OPT_MODE].s, "Lineart") != 0)) ? 12 : s->bpp;
+	&& (strcmp (s->val[OPT_MODE].s, SANE_I18N("Lineart")) != 0))
+	? 12 : s->bpp;
 #endif
       wbuf[36] = 0;
       wbuf[37] = (s->RIF << 7) + 0x3;
@@ -1502,7 +1484,7 @@ sane_start (SANE_Handle handle)
       if (!s->val[OPT_PREVIEW].w && s->hw->info.is_filmscanner)
 	wbuf[11] = s->scanning_speed;
       wbuf[15] = (s->hw->info.model == FB620
-	&& !strcmp (mode_str, "Fine color")
+	&& !strcmp (mode_str, SANE_I18N("Fine color"))
 	&& !s->val[OPT_PREVIEW].w) ? 1 << 3 : 0;
       status = define_scan_mode (s->fd, SCAN_CONTROL_CONDITIONS, wbuf);
     }
@@ -1524,7 +1506,7 @@ sane_start (SANE_Handle handle)
       return (SANE_STATUS_INVAL);
     }
   for (i = 0; i < buf_size; i++)
-    DBG (3, "scan mode byte[%d] = %d\n", i, ebuf[i]);
+    DBG (3, "scan mode byte[%d] = %d\n", (int) i, ebuf[i]);
 
   /* Focus, but not for previews or negatives with speed control */
   if (SANE_OPTION_IS_ACTIVE(s->opt[OPT_FOCUS_GROUP].cap)
@@ -1638,7 +1620,7 @@ sane_start (SANE_Handle handle)
 	  s->bytes_to_read = s->params.bytes_per_line * s->params.lines;
 
 	  mode_str = s->val[OPT_MODE].s;
-	  if (!strcmp (mode_str, "Lineart"))
+	  if (!strcmp (mode_str, SANE_I18N("Lineart")))
 	    {
 	      if (((600 < s->val[OPT_X_RESOLUTION].w)
 		|| (600 < s->val[OPT_Y_RESOLUTION].w)))
@@ -1648,10 +1630,10 @@ sane_start (SANE_Handle handle)
 		}
 	      s->params.pixels_per_line = s->params.bytes_per_line * 8;
 	    }
-	  else if (!strcmp (mode_str, "Gray"))
+	  else if (!strcmp (mode_str, SANE_I18N("Gray")))
 	      s->params.pixels_per_line = s->params.bytes_per_line;
-	  else if (!strcmp (mode_str, "Color")
-	    || !strcmp (mode_str, "Fine color"))
+	  else if (!strcmp (mode_str, SANE_I18N("Color"))
+	    || !strcmp (mode_str, SANE_I18N("Fine color")))
 	      s->params.pixels_per_line = s->params.bytes_per_line / 3;
 	  else
 	    s->params.pixels_per_line = s->params.bytes_per_line / 6;
@@ -1690,8 +1672,8 @@ sane_read_direct (SANE_Handle handle, SANE_Byte *buf, SANE_Int max_len,
   *len = 0;
   nread = max_len;
 
-  DBG (21, "   sane_read: nread=%d, bytes_to_read=%d\n", nread,
-       s->bytes_to_read);
+  DBG (21, "   sane_read: nread=%d, bytes_to_read=%d\n", (int) nread,
+  (int) s->bytes_to_read);
   if (s->bytes_to_read == 0)
     {
       do_cancel (s);
@@ -1710,8 +1692,8 @@ sane_read_direct (SANE_Handle handle, SANE_Byte *buf, SANE_Int max_len,
   *len = nread;
   s->bytes_to_read -= nread;
 
-  DBG (21, "   sane_read: nread=%d, bytes_to_read=%d\n", nread,
-       s->bytes_to_read);
+  DBG (21, "   sane_read: nread=%d, bytes_to_read=%d\n", (int) nread,
+  (int) s->bytes_to_read);
   DBG (21, "<< sane_read\n");
   return (SANE_STATUS_GOOD);
 }
@@ -1726,15 +1708,18 @@ read_fs2710 (SANE_Handle handle, SANE_Byte *buf, SANE_Int max_len,
   SANE_Status status;
   int c;
   size_t i, nread, nread2;
-  u_char *p, b;
+  u_char *p;
+#if defined(WORDS_BIGENDIAN)
+  u_char b;
+#endif
 
   DBG (21, ">> sane_read\n");
 
   *len = 0;
   nread = max_len;
 
-  DBG (21, "   sane_read: nread=%d, bytes_to_read=%d\n", nread,
-       s->bytes_to_read);
+  DBG (21, "   sane_read: nread=%d, bytes_to_read=%d\n", (int) nread,
+  (int) s->bytes_to_read);
 
   if (nread > s->bytes_to_read) nread = s->bytes_to_read;
   if (s->bytes_to_read == 0)
@@ -1745,13 +1730,14 @@ read_fs2710 (SANE_Handle handle, SANE_Byte *buf, SANE_Int max_len,
 
   if (!s->scanning) return (do_cancel (s));
 
-  /* We must receive 2 bytes per pixel and colour. In raw mode we
-     must swap the bytes (for compatibility with xsane) and pass
-     them both. Otherwise the other subroutines expect only 1 byte,
-     so we must set up an intermediate buffer which is twice as large
+  /* We must receive 2 little-endian bytes per pixel and colour.
+     In raw mode we must swap the bytes if we are running a big-endian
+     architecture (SANE standard 3.2.1), and pass them both.
+     Otherwise the other subroutines expect only 1 byte, so we must
+     set up an intermediate buffer which is twice as large
      as buf, and then map this buffer to buf. */
 
-  if (!strcmp (s->val[OPT_MODE].s, "Color"))
+  if (!strcmp (s->val[OPT_MODE].s, SANE_I18N("Color")))
     {
       if (max_len > s->auxbuf_len)
 	{				/* extend buffer? */
@@ -1787,18 +1773,21 @@ read_fs2710 (SANE_Handle handle, SANE_Byte *buf, SANE_Int max_len,
 	  do_cancel (s);
 	  return (SANE_STATUS_IO_ERROR);
 	}
+#if defined(WORDS_BIGENDIAN)
       for (p = buf; p < buf + nread; p++)
 	{
 	  b = *p;
-	  *p++ = *(p + 1);
+	  *p = *(p + 1);
+	  p++;
 	  *p = b;
 	}
+#endif
     }
   *len = nread;
   s->bytes_to_read -= nread;
 
-  DBG (21, "   sane_read: nread=%d, bytes_to_read=%d\n", nread,
-       s->bytes_to_read);
+  DBG (21, "   sane_read: nread=%d, bytes_to_read=%d\n", (int) nread,
+  (int) s->bytes_to_read);
   DBG (21, "<< sane_read\n");
   return (SANE_STATUS_GOOD);
 }
@@ -1820,8 +1809,8 @@ read_fb620 (SANE_Handle handle, SANE_Byte *buf, SANE_Int max_len,
 
   *len = 0;
 
-  DBG (21, "   read_fb620: nread=%d, bytes_to_read=%d\n", nread,
-       s->bytes_to_read);
+  DBG (21, "   read_fb620: nread=%d, bytes_to_read=%d\n", (int) nread,
+  (int) s->bytes_to_read);
 
   if (s->bytes_to_read == 0 && s->buf_pos == s->buf_used)
     {
@@ -1919,9 +1908,10 @@ SANE_Int *len)
 {
   CANON_Scanner *s = handle;
   SANE_Status status;
-  SANE_Byte *firstimage, *secondimage/*, inmask, outmask, outbyte, primaryHigh[256], primaryLow[256], secondaryHigh[256], secondaryLow[256] */;
+  SANE_Byte *firstimage, *secondimage/*, inmask, outmask, outbyte,
+	    primaryHigh[256], primaryLow[256], secondaryHigh[256],
+	    secondaryLow[256] */;
   SANE_Int ncopy;
-  
   u_char dbuf[28];
   size_t buf_size, nread, remain, nwritten, nremain, pos, pix, pixel_per_line,
 	 byte, byte_per_line/*, bit*/;
@@ -1945,13 +1935,14 @@ SANE_Int *len)
     + (dbuf[22] << 8) + dbuf[23]);
   DBG (5, "Filled Data Buffer=%d\n", (dbuf[24] << 24) + (dbuf[25] << 16)
     + (dbuf[26] << 8) + dbuf[27]);
-  DBG (5, "temp file position:%u\n", lseek(s->tmpfile, 0, SEEK_CUR));
+  DBG (5, "temp file position:%u\n", (unsigned int) lseek(s->tmpfile,
+  0, SEEK_CUR));
   DBG (5, "<< GET DATA STATUS\n");
 
   *len = 0;
 
-  DBG (21, "   read_fb1200: nread=%d, bytes_to_read=%d\n", nread,
-       s->bytes_to_read);
+  DBG (21, "   read_fb1200: bytes_to_read=%d\n",
+  (int) s->bytes_to_read);
 
   if (s->bytes_to_read == 0 && s->buf_pos == s->buf_used)
     {
@@ -2063,7 +2054,7 @@ SANE_Int *len)
 
       secondimage = s->inbuffer;
 
-      if (!strcmp (s->val[OPT_MODE].s, "Color"))
+      if (!strcmp (s->val[OPT_MODE].s, SANE_I18N("Color")))
 	{
 	  maxpix = pixel_per_line / 2;
 	  for (pix = 0; (int) pix < maxpix; pix++)
@@ -2076,7 +2067,7 @@ SANE_Int *len)
 	      s->outbuffer[6 * pix + 5] = firstimage[3 * pix + 2];
 	    }
 	}
-      else if (!strcmp (s->val[OPT_MODE].s, "Gray"))
+      else if (!strcmp (s->val[OPT_MODE].s, SANE_I18N("Gray")))
 	{
 	  for (pix = 0; pix < pixel_per_line / 2; pix++)
 	    {
@@ -2201,7 +2192,8 @@ sane_cancel (SANE_Handle handle)
 /**************************************************************************/
 
 SANE_Status
-sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
+sane_set_io_mode (SANE_Handle UNUSEDARG handle,
+SANE_Bool UNUSEDARG non_blocking)
 {
   DBG (1, ">> sane_set_io_mode\n");
   DBG (1, "<< sane_set_io_mode\n");
@@ -2211,7 +2203,8 @@ sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
 /**************************************************************************/
 
 SANE_Status
-sane_get_select_fd (SANE_Handle handle, SANE_Int * fd)
+sane_get_select_fd (SANE_Handle UNUSEDARG handle,
+SANE_Int UNUSEDARG * fd)
 {
   DBG (1, ">> sane_get_select_fd\n");
   DBG (1, "<< sane_get_select_fd\n");
