@@ -154,7 +154,7 @@ done only once, at first scan after powerup.
 static SANE_Word optionCalibrateValue = SANE_FALSE;
 
 static SANE_Option_Descriptor optionCalibrateDescriptor = {
-  SANE_I18N ("cal"),
+  "cal",
   SANE_I18N ("Calibrate Scanner"),
   SANE_I18N ("Force scanner calibration before scan"),
   SANE_TYPE_BOOL,
@@ -254,7 +254,7 @@ This option lets the user select a gray scale scan
 static SANE_Word optionGrayscaleValue = SANE_FALSE;
 
 static SANE_Option_Descriptor optionGrayscaleDescriptor = {
-  SANE_I18N ("gray"),
+  "gray",
   SANE_I18N ("Grayscale scan"),
   SANE_I18N ("Do a grayscale rather than color scan"),
   SANE_TYPE_BOOL,
@@ -301,7 +301,7 @@ static const SANE_Range aGainRange = {
 static SANE_Int optionAGainValue = 1;
 
 static SANE_Option_Descriptor optionAGainDescriptor = {
-  SANE_I18N ("gain"),
+  "gain",
   SANE_I18N ("Analog Gain"),
   SANE_I18N ("Increase or decrease the analog gain of the CCD array"),
   SANE_TYPE_INT,
@@ -342,7 +342,7 @@ optionAGainCallback (SANE_Option * option, SANE_Handle handle,
 static SANE_Fixed optionGammaValue = SANE_FIX (1.6);
 
 static SANE_Option_Descriptor optionGammaDescriptor = {
-  SANE_I18N ("gamma"),
+  "gamma",
   SANE_I18N ("Gamma Correction"),
   SANE_I18N ("Selects the gamma corrected transfer curve"),
   SANE_TYPE_FIXED,
@@ -485,10 +485,10 @@ optionTopLeftYCallback (SANE_Option * option, SANE_Handle handle,
 /*-----------------------------------------------------------------*/
 /*
 This option controls the bot-right-x corner of the scan
-Default to 2 inches.
+Default to 215.9mm, max.
 */
 
-static SANE_Fixed optionBotRightXValue = SANE_FIX (2 * MM_IN_INCH);
+static SANE_Fixed optionBotRightXValue = SANE_FIX (215.9);
 
 static SANE_Option_Descriptor optionBotRightXDescriptor = {
   SANE_NAME_SCAN_BR_X,
@@ -529,10 +529,10 @@ optionBotRightXCallback (SANE_Option * option, SANE_Handle handle,
 /*-----------------------------------------------------------------*/
 /*
 This option controls the bot-right-y corner of the scan
-Default to 2 inches
+Default to 296.3mm, max
 */
 
-static SANE_Fixed optionBotRightYValue = SANE_FIX (2 * MM_IN_INCH);
+static SANE_Fixed optionBotRightYValue = SANE_FIX (296.3);
 
 static SANE_Option_Descriptor optionBotRightYDescriptor = {
   SANE_NAME_SCAN_BR_Y,
@@ -930,7 +930,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 
   DBG (3,
        "sane_control_option: handle=%p, opt=%d, act=%d, val=%p, info=%p\n",
-       handle, option, action, value, info);
+       handle, option, action, value, (void *)info);
 
   return dispatch_control_option (handle, option, action, value, info);
 }
@@ -1037,9 +1037,7 @@ sane_set_io_mode (SANE_Handle handle, SANE_Bool non_blocking)
 SANE_Status
 sane_get_select_fd (SANE_Handle handle, SANE_Int * fd)
 {
-  Canon_Scanner *scanner = handle;
-  DBG (3, "sane_get_select_fd: handle = %p, fd %s 0\n", handle,
-       fd ? "!=" : "=");
-  *fd = scanner->scan.fd;
-  return SANE_STATUS_GOOD;
+  handle = handle;                   /* silence gcc */
+  fd = fd;                           /* silence gcc */
+  return SANE_STATUS_UNSUPPORTED;
 }
