@@ -1482,6 +1482,7 @@ sane_read (SANE_Handle h, SANE_Byte * buf, SANE_Int maxlen, SANE_Int * len)
       if (xfer_len_out == 0)	/* no more data */
 	{
 	  *len = 0;
+	s->scanning = SANE_FALSE;
 	  return SANE_STATUS_EOF;
 	}
 
@@ -1524,7 +1525,7 @@ sane_read (SANE_Handle h, SANE_Byte * buf, SANE_Int maxlen, SANE_Int * len)
       xfer_len_in += s->block_padding;
       i = (xfer_len_in & 0x3f);
       if (i != 0)
-        DBG (1, "BUG: sane_read(): Read size is not a multiple of 64. (0x%06x)\n", i);
+        DBG (1, "BUG: sane_read(): Read size is not a multiple of 64. (0x%06lx)\n", (long) i);
     }
 
   if (s->xfer_position + xfer_len_line > s->xfer_bytes_total)
@@ -1533,6 +1534,7 @@ sane_read (SANE_Handle h, SANE_Byte * buf, SANE_Int maxlen, SANE_Int * len)
   if (xfer_len_line == 0)	/* no more data */
     {
       *len = 0;
+	s->scanning = SANE_FALSE;
       return SANE_STATUS_EOF;
     }
 
@@ -1770,11 +1772,11 @@ cs2_open (const char *device, cs2_interface_t interface, cs2_t ** sp)
       return status;
     }
 
-  strncpy (s->vendor_string, s->recv_buf + 8, 8);
+  strncpy (s->vendor_string, (char *)s->recv_buf + 8, 8);
   s->vendor_string[8] = '\0';
-  strncpy (s->product_string, s->recv_buf + 16, 16);
+  strncpy (s->product_string, (char *)s->recv_buf + 16, 16);
   s->product_string[16] = '\0';
-  strncpy (s->revision_string, s->recv_buf + 32, 4);
+  strncpy (s->revision_string, (char *)s->recv_buf + 32, 4);
   s->revision_string[4] = '\0';
 
   DBG (10,
