@@ -3,7 +3,7 @@
 
    dc25.c 
 
-   $Id: dc25.c,v 1.15 2003/01/07 01:10:44 psfales Exp $
+   $Id: dc25.c,v 1.17 2008-02-03 10:34:15 ellert-guest Exp $
 
    This file (C) 1998 Peter Fales
 
@@ -1850,7 +1850,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback UNUSEDARG authorize)
   fp = sanei_config_open (DC25_CONFIG_FILE);
 
   DBG (127,
-       "sane_init() $Id: dc25.c,v 1.15 2003/01/07 01:10:44 psfales Exp $\n");
+       "sane_init() $Id: dc25.c,v 1.17 2008-02-03 10:34:15 ellert-guest Exp $\n");
 
   if (!fp)
     {
@@ -1907,7 +1907,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback UNUSEDARG authorize)
 		  tty_baud = DEFAULT_TTY_BAUD;
 		  break;
 		}
-	      DBG (20, "Config file baud=%d\n", tty_baud);
+	      DBG (20, "Config file baud=%lu\n", (u_long) tty_baud);
 	    }
 	  else if (strcmp (dev_name, "dumpinquiry") == 0)
 	    {
@@ -2337,7 +2337,7 @@ sane_start (SANE_Handle handle)
   int n, i;
   FILE *f;
 
-  DBG (127, "sane_start called, handle=%x\n",(unsigned int)handle);
+  DBG (127, "sane_start called, handle=%lx\n", (u_long) handle);
 
   if (handle != MAGIC || !is_open ||
       (dc25_opt_image_number == 0 && dc25_opt_snap == SANE_FALSE))
