@@ -2,7 +2,7 @@
 
    Copyright (C) 2002 Sergey Vlasov <vsu@altlinux.ru>
    AFE offset/gain setting by David Stevenson <david.stevenson@zoom.co.uk>
-   Copyright (C) 2002 - 2005 Henning Meier-Geinitz <henning@meier-geinitz.de>
+   Copyright (C) 2002 - 2007 Henning Geinitz <sane@geinitz.org>
 
    This file is part of the SANE package.
    
@@ -400,13 +400,27 @@ gt68xx_scanner_wait_for_positioning (GT68xx_Scanner * scanner)
 {
   SANE_Status status;
   SANE_Bool moving;
+  SANE_Int status_count = 0;
 
   usleep (100000);		/* needed by the BP 2400 CU Plus? */
   while (SANE_TRUE)
     {
-      RIE (gt68xx_device_is_moving (scanner->dev, &moving));
-      if (!moving)
-	break;
+      status = gt68xx_device_is_moving (scanner->dev, &moving);
+      if (status == SANE_STATUS_GOOD)
+	{
+	  if (!moving)
+	    break;
+	}
+      else
+	{
+	  if (status_count > 9)
+	    {
+	      DBG (1, "gt68xx_scanner_wait_for_positioning: error count too high!\n");
+	      return status;
+	    }
+	  status_count++;
+	  DBG(3, "gt68xx_scanner_wait_for_positioning: ignored error\n");
+	}
       usleep (100000);
     }
 
@@ -510,7 +524,8 @@ gt68xx_scanner_start_scan_extended (GT68xx_Scanner * scanner,
       return status;
     }
 
-  if (scanner->dev->model->is_cis)
+  if (scanner->dev->model->is_cis 
+      && !(scanner->dev->model->flags & GT68XX_FLAG_SHEET_FED))
     {
       status =
 	gt68xx_device_set_exposure_time (scanner->dev,
@@ -681,7 +696,7 @@ gt68xx_scanner_calibrate (GT68xx_Scanner * scanner,
   req.mds = SANE_TRUE;
   req.mas = SANE_FALSE;
 
-  if (scanner->dev->model->is_cis)
+  if (scanner->dev->model->is_cis && !(scanner->dev->model->flags & GT68XX_FLAG_CIS_LAMP))
     req.color = SANE_TRUE;
 
   if (req.use_ta)
@@ -897,7 +912,7 @@ gt68xx_scanner_read_line (GT68xx_Scanner * scanner,
 	}
       else
 	{
-	  if (scanner->dev->model->is_cis)
+	  if (scanner->dev->model->is_cis && !(scanner->dev->model->flags & GT68XX_FLAG_CIS_LAMP))
 	    {
 	      if (strcmp
 		  (scanner->val[OPT_GRAY_MODE_COLOR].s,
@@ -1145,16 +1160,18 @@ gt68xx_wait_lamp_stable (GT68xx_Scanner * scanner,
 			 GT68xx_Afe_Values *values,
 			 SANE_Bool dont_move)
 {
-  int i;
   SANE_Status status = SANE_STATUS_GOOD;
   SANE_Int last_white = 0;
-  SANE_Int first = SANE_TRUE;
+  SANE_Bool first = SANE_TRUE;
+  SANE_Bool message_printed = SANE_FALSE;
+  struct timeval now, start_time;
+  int secs_lamp_on, secs_start;
+  int increase = -5;
 
-  for (i = 0; i < 80; i++)
+  gettimeofday (&start_time, 0);
+  do
     {
       usleep (200000);
-      if (i == 10)
-	DBG (0, "Please wait for lamp warm-up\n");
 
       if (!first && dont_move)
 	{
@@ -1189,18 +1206,25 @@ gt68xx_wait_lamp_stable (GT68xx_Scanner * scanner,
 	   "gt68xx_wait_lamp_stable: this white = %d, last white = %d\n",
 	   values->total_white, last_white);
 
+      gettimeofday (&now, 0);
+      secs_lamp_on = now.tv_sec - scanner->lamp_on_time.tv_sec;
+      secs_start = now.tv_sec - start_time.tv_sec;
+
+      if (!message_printed && secs_start > 5)
+	{
+	  DBG (0, "Please wait for lamp warm-up\n");
+	  message_printed = SANE_TRUE;
+	}
 
       if (scanner->val[OPT_AUTO_WARMUP].w == SANE_TRUE)
 	{
 	  if (scanner->dev->model->flags & GT68XX_FLAG_CIS_LAMP)
 	    {
-	      /* insist on at least 10 seconds */
-	      struct timeval now;
-	      int secs;
-
-	      gettimeofday (&now, 0);
-	      secs = now.tv_sec - scanner->lamp_on_time.tv_sec;
-	      if (secs >= 10 && (values->total_white <= (last_white + 20))
+	      if (values->total_white <= (last_white - 20))
+		  increase--;
+	      if (values->total_white >= last_white)
+		  increase++;
+	      if (increase > 0 && (values->total_white <= (last_white + 20))
 		  && values->total_white != 0)
 		break;
 	    }
@@ -1211,19 +1235,12 @@ gt68xx_wait_lamp_stable (GT68xx_Scanner * scanner,
 		break;		/* lamp is warmed up */
 	    }
 	}
-      else
-	{			/* insist on 60 seconds */
-	  struct timeval now;
-	  int secs;
-
-	  gettimeofday (&now, 0);
-	  secs = now.tv_sec - scanner->lamp_on_time.tv_sec;
-	  if (secs >= WARMUP_TIME)
-	    break;
-	}
       last_white = values->total_white;
     }
-  DBG (3, "gt68xx_wait_lamp_stable: Lamp is stable\n");
+  while (secs_lamp_on <= WARMUP_TIME);
+
+  DBG (3, "gt68xx_wait_lamp_stable: Lamp is stable after %d secs (%d secs total)\n",
+       secs_start, secs_lamp_on);
   return status;
 }
 
@@ -1705,6 +1722,7 @@ gt68xx_afe_cis_read_lines (GT68xx_Afe_Values * values,
       values->scan_dpi = params.xdpi;
       values->calwidth = params.pixel_xs;
       values->callines = params.pixel_ys;
+      values->start_black = scanner->dev->model->x_offset_mark;
       return SANE_STATUS_GOOD;
     }
 
