@@ -1,7 +1,7 @@
 /* sane - Scanner Access Now Easy.
 
    Copyright (C) 2002 Sergey Vlasov <vsu@altlinux.ru>
-   Copyright (C) 2002 - 2004 Henning Meier-Geinitz <henning@meier-geinitz.de>
+   Copyright (C) 2002 - 2007 Henning Geinitz <sane@geinitz.org>
    
    This file is part of the SANE package.
    
@@ -113,6 +113,7 @@
 #define GT68XX_FLAG_SCAN_FROM_HOME  (1 << 9)	/* Move home after calibration */
 #define GT68XX_FLAG_USE_OPTICAL_X   (1 << 10)	/* Use optical xdpi for 50 dpi and below */
 #define GT68XX_FLAG_ALWAYS_LINEMODE (1 << 11)	/* Linemode must be used for any resolution */
+#define GT68XX_FLAG_SHEET_FED       (1 << 12)	/* we have a sheet fed scanner */
 
 /* Forward typedefs */
 typedef struct GT68xx_USB_Device_Entry GT68xx_USB_Device_Entry;
@@ -157,7 +158,7 @@ struct GT68xx_USB_Device_Entry
   GT68xx_Model *model;			/**< Scanner model information */
 };
 
-#define MAX_SCANNERS 30
+#define MAX_SCANNERS 50
 /* Looks like gcc doesn't like declarations without specificating the number
    of array elements at least when --enable-warnings is active */
 
@@ -333,6 +334,7 @@ struct GT68xx_Command_Set
    */
     SANE_Status (*is_moving) (GT68xx_Device * dev, SANE_Bool * moving);
 
+
   /** Move the scanner carriage by the specified number of steps.
    *
    * @param dev Device object.
@@ -353,6 +355,12 @@ struct GT68xx_Command_Set
    */
     SANE_Status (*carriage_home) (GT68xx_Device * dev);
 
+  /** Eject the paper at the end of the scan.
+   *
+   * @param dev Device object.
+   */
+    SANE_Status (*paperfeed) (GT68xx_Device * dev);
+
   /** Start scanning the image.
    *
    * @param dev Device object.
@@ -903,6 +911,17 @@ gt68xx_device_move_relative (GT68xx_Device * dev, SANE_Int distance);
  */
 static SANE_Status gt68xx_device_carriage_home (GT68xx_Device * dev);
 
+/** Eject the paper after the end of scanning.
+ *
+ *
+ * @param dev Device object.
+ *
+ * @return
+ * - #SANE_STATUS_GOOD - success; the movement is started.
+ * - #SANE_STATUS_IO_ERROR - a communication error occured.
+ */
+static SANE_Status gt68xx_device_paperfeed (GT68xx_Device * dev);
+
 /** Start scanning the image.
  *
  * This function initiates scanning with parameters set by
@@ -1038,7 +1057,20 @@ static SANE_Status
 gt68xx_device_check_result (GT68xx_Packet res, SANE_Byte command);
 
 
-static SANE_Status gt68xx_device_get_id (GT68xx_Device * dev);
+static SANE_Status 
+gt68xx_device_get_id (GT68xx_Device * dev);
+
+/** Read the device descriptor of the scanner.
+ *
+ * This function should be called before closing the device to make sure
+ * that the device descriptor is propperly stored in the scanner's memory.
+ * If that's not done, the next try to get the config descriptor will
+ * result in a corrupted descriptor.
+ *
+ * @param dev device
+*/
+static void 
+gt68xx_device_fix_descriptor (GT68xx_Device * dev);
 
 #endif /* not GT68XX_LOW_H */
 
