@@ -89,8 +89,8 @@
 #define MAX(a,b)                ((a) > (b)) ? (a) : (b)
 
 #define MICROTEK2_MAJOR         0
-#define MICROTEK2_MINOR	        95
-#define MICROTEK2_BUILD         "200301152155"
+#define MICROTEK2_MINOR	        96
+#define MICROTEK2_BUILD         "200410042220"
 #define MICROTEK2_CONFIG_FILE   "microtek2.conf"
 
 
@@ -526,7 +526,6 @@ enum Microtek2_Option
   OPT_MODE,
   OPT_BITDEPTH,
   OPT_RESOLUTION,
-  OPT_X_RESOLUTION,
   OPT_Y_RESOLUTION,
   OPT_PREVIEW,
 
@@ -1287,8 +1286,8 @@ read_cx_shading_image(Microtek2_Scanner *);
 static SANE_Status
 read_cx_shading(Microtek2_Scanner *);
 
-static SANE_Status
-reader_process(Microtek2_Scanner *);
+static int
+reader_process(void *);
 
 static SANE_Status
 restore_gamma_options(SANE_Option_Descriptor *, Option_Value *);
@@ -1344,7 +1343,7 @@ scsi_read_control_bits(Microtek2_Scanner *);
 scsi_read_gamma(Microtek2_Scanner *); */
 
 static SANE_Status
-scsi_read_image(Microtek2_Scanner *, u_int8_t *);
+scsi_read_image(Microtek2_Scanner *, u_int8_t *, int);
 
 static SANE_Status
 scsi_read_image_info(Microtek2_Scanner *);
