@@ -7,7 +7,7 @@
  *  @brief Main defines for the USB devices.
  *
  * Based on sources acquired from Plustek Inc.<br>
- * Copyright (C) 2001-2006 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2001-2007 Gerhard Jaeger <gerhard@gjaeger.de>
  *
  * History:
  * - 0.40 - starting version of the USB support
@@ -44,6 +44,14 @@
  *        - added _WAF_USE_ALT_DESC
  *        - added DEVCAPSFLAG_SheetFed
  *        - added dpi_thresh and lineend to motor structure
+ * - 0.52 - added MODEL_QSCAN
+ *        - changed DCapsDef, lamp -> misc_io
+ *        - bPCB is now ushort to be "missused" by non Plustek
+ *          devices (as threshhold for resetting sensor order)
+ *        - added _WAF_LOFF_ON_START and _WAF_ONLY_8BIT
+ *        - added MODEL_TSCAN_A4
+ *        - added attribute packed for data access structs
+ *        - added _WAF_INC_DARKTGT
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -117,20 +125,26 @@
 #define _HIBYTE(x)  ((u_char)((x) >> 8))
 
 #define _HILO2WORD(x)   ((u_short)x.bHi * 256U + x.bLo)
+#define _LOHI2WORD(x)   ((u_short)x.bLo * 256U + x.bHi)
+
 #define _PHILO2WORD(x)  ((u_short)x->bHi * 256U + x->bLo)
+#define _PLOHI2WORD(x)  ((u_short)x->bLo * 256U + x->bHi)
+
+#define PACKED8  __attribute__ ((packed,aligned(1)))
+#define PACKED16 __attribute__ ((packed,aligned(2)))
 
 /* useful for RGB-values */
 typedef struct {
 	u_char Red;
 	u_char Green;
 	u_char Blue;
-} RGBByteDef;
+} PACKED8 RGBByteDef;
 
 typedef struct {
 	u_short Red;
 	u_short Green;
 	u_short Blue;
-} RGBUShortDef;
+} PACKED16 RGBUShortDef;
 
 typedef struct {
 	u_long Red;
@@ -140,22 +154,22 @@ typedef struct {
 
 typedef struct {
 	u_char a_bColor[3];
-} ColorByteDef;
+} PACKED8 ColorByteDef;
 
 typedef struct {
 	u_char bHi;
 	u_char bLo;
-} HiLoDef;
+} PACKED8 HiLoDef;
 
 typedef union {
 	HiLoDef HiLo[3];
 	u_short Colors[3];
-} ColorWordDef;
+} PACKED16 ColorWordDef;
 
 typedef union {
 	HiLoDef HiLo;
 	u_short Mono;
-} MonoWordDef;
+} PACKED16 MonoWordDef;
 
 typedef union {
 
@@ -168,7 +182,7 @@ typedef union {
 	RGBUShortDef *pw_rgb;
 	HiLoDef      *philo;
 
-} AnyPtr;
+} __attribute__ ((aligned(4)))  AnyPtr;
 
 typedef struct {
 	unsigned short x;
@@ -248,7 +262,11 @@ enum _WORKAROUNDS
 	_WAF_BIN_FROM_COLOR     = 0x00000080, /* generate binary & gray images   */
 	_WAF_GRAY_FROM_COLOR    = 0x00000100, /* from color scans                */
 	_WAF_MISC_IO_BUTTONS    = 0x00000200, /* special handling for buttons    */
-	_WAF_USE_ALT_DESC       = 0x00000400  /* use alternate manufacturer      */
+	_WAF_USE_ALT_DESC       = 0x00000400, /* use alternate manufacturer      */
+	_WAF_RESET_SO_TO_RGB    = 0x00000800, /* set sensororder to RGB(CIS only)*/
+	_WAF_LOFF_ON_START      = 0x00001000, /* switch lamp off before scanning */
+	_WAF_ONLY_8BIT          = 0x00002000, /* scanner allows only 8 bit modes */
+	_WAF_INC_DARKTGT        = 0x00004000  /* allow tgt darklevel adjustment  */
 };
 
 /** for lamps connected to the misc I/O pins*/
@@ -263,6 +281,8 @@ enum _LAMPS
 	_MIO6   = 0x0020
 };
 
+#define _PSENSE_SHIFT 24
+#define _PSENSE_MASK  0xFF000000
 #define _BUTTON_SHIFT 16
 #define _BUTTON_MASK  0xFF0000
 #define _TPA_SHIFT    8
@@ -276,6 +296,18 @@ enum _BUTTONS
 	_PORT2     = ((_MIO5 | _MIO6) << _BUTTON_SHIFT)
 };
 
+enum _PAPER_SENSE
+{
+	_PS_INP1     = (0x01  << _PSENSE_SHIFT),
+	_PS_INP2     = (0x02  << _PSENSE_SHIFT),
+	_PS_INP_MIO1 = (_MIO1 << (_PSENSE_SHIFT+2)),
+	_PS_INP_MIO2 = (_MIO2 << (_PSENSE_SHIFT+2)),
+	_PS_INP_MIO3 = (_MIO3 << (_PSENSE_SHIFT+2)),
+	_PS_INP_MIO4 = (_MIO4 << (_PSENSE_SHIFT+2)),
+	_PS_INP_MIO5 = (_MIO5 << (_PSENSE_SHIFT+2)),
+	_PS_INP_MIO6 = (_MIO6 << (_PSENSE_SHIFT+2))
+};
+
 /** for encoding a misc I/O register as TPA */
 #define _TPA(register) ((u_long)(register << _TPA_SHIFT))
 
@@ -285,6 +317,9 @@ enum _BUTTONS
 /** Get the TPA misc I/O register */
 #define _GET_TPALAMP(flag) ((flag >> _TPA_SHIFT) & 0xFF)
 
+/** Get the Papersense port*/
+#define _GET_PAPERSENSE_PORT(flag) ((flag >> _PSENSE_SHIFT) & 0xFF)
+
 /** motor types */
 typedef enum
 {
@@ -301,7 +336,9 @@ typedef enum
 	MODEL_CANON_LIDE25,   /**< for CanoScan LiDE25          */
 	MODEL_UMAX,           /**< for UMAX 3400/3450           */
 	MODEL_UMAX1200,       /**< for UMAX 5400                */
-	MODEL_TSCAN,          /**< for Syscan Travelscan        */
+	MODEL_TSCAN,          /**< for Syscan TravelScan A6     */
+	MODEL_TSCAN_A4,       /**< for Syscan TravelScan A4     */
+	MODEL_QSCAN,          /**< for PandP Q-Scan             */
 	MODEL_LAST
 } eModelDef;
 
@@ -382,10 +419,10 @@ typedef struct DevCaps
 	u_char     bSensorDistance; /**< CCD Color distance                      */
 	u_char     bButtons;        /**< Number of buttons                       */
 	u_char     bCCD;            /**< CCD ID                                  */
-	u_char     bPCB;            /**< PCB ID                                  */
+	u_short    bPCB;            /**< PCB ID/or threshold (only CIS)          */
 	u_long     workaroundFlag;  /**< Flag to allow special work arounds, see */
 	                            /*   _WORKAROUNDS                            */
-	u_long     lamp;            /**< for lamp: loword: normal, hiword: tpa   */
+	u_long     misc_io;         /**< for lamp, papersense and buttons        */
 
 } DCapsDef;
 
@@ -484,7 +521,7 @@ typedef struct HWDefault
 
 	eChipDef            chip;           /* chiptype               */
 	eModelDef           motorModel;     /* to identify used motor */
-	double              gamma;          /* gamma setting          */
+	double              gamma;          /* default gamma setting  */
 } HWDef;
 
 /** device description during runtime
@@ -598,7 +635,7 @@ typedef struct ScanDef
 	/** Image processing routine according to the scan mode  */
 	void (*pfnProcess)(struct Plustek_Device*);
 
-	u_char* pScanBuffer;      /**< our scan buffer */
+	u_long* pScanBuffer;      /**< our scan buffer */
 
 	u_long  dwLinesPerScanBufs;
 	u_long  dwNumberOfScanBufs;
