@@ -83,7 +83,7 @@
 #include "../include/sane/sanei.h"
 #include "../include/sane/saneopts.h"
 
-#define BACKEND_VERSION "0.43-10"
+#define BACKEND_VERSION "0.43-13"
 #define BACKEND_NAME	plustek_pp
 #include "../include/sane/sanei_backend.h"
 #include "../include/sane/sanei_config.h"
@@ -468,7 +468,7 @@ static SANE_Status do_cancel( Plustek_Scanner *scanner, SANE_Bool closepipe  )
 
 	scanner->scanning = SANE_FALSE;
 
-	if( scanner->reader_pid > 0 ) {
+	if( scanner->reader_pid != -1 ) {
 
 		DBG( _DBG_PROC, ">>>>>>>> killing reader_process <<<<<<<<\n" );
 
@@ -503,7 +503,7 @@ static SANE_Status do_cancel( Plustek_Scanner *scanner, SANE_Bool closepipe  )
 #endif
 		}			
 
-		scanner->reader_pid = 0;
+		scanner->reader_pid = -1;
 		DBG( _DBG_PROC,"reader_process killed\n");
 	}
 
@@ -1011,9 +1011,9 @@ static SANE_Status attach( const char *dev_name, pCnfDef cnf,
 
 	show_cnf( cnf );
 
-	dev->sane.type   = "parallel port flatbed scanner";
-	dev->open  	     = ppDev_open;
-	dev->close 	     = ppDev_close;
+	dev->sane.type   = SANE_I18N ("flatbed scanner");
+	dev->open        = ppDev_open;
+	dev->close       = ppDev_close;
 	dev->getCaps     = ppDev_getCaps;
 	dev->getLensInfo = ppDev_getLensInfo;
 	dev->getCropInfo = ppDev_getCropInfo;
@@ -2014,9 +2014,9 @@ SANE_Status sane_start( SANE_Handle handle )
 	s->w_pipe     = fds[1];
 	s->reader_pid = sanei_thread_begin( reader_process, s );
 
-	if( s->reader_pid < 0 ) {
+	if( s->reader_pid == -1 ) {
 		DBG( _DBG_ERROR, "ERROR: could not create child process\n" );
-	    s->scanning = SANE_FALSE;
+		s->scanning = SANE_FALSE;
 		s->hw->close( s->hw );
 		return SANE_STATUS_IO_ERROR;
 	}
