@@ -40,7 +40,7 @@
 */
 
 /*
-	$Id: ricoh.c,v 1.4 2002/08/23 03:04:01 fzago Exp $
+	$Id: ricoh.c,v 1.7 2008-02-04 01:20:01 kitno-guest Exp $
         This file implements a SANE backend for Ricoh flatbed scanners.  
 */
 
@@ -58,7 +58,6 @@
 #include "sane/sane.h"
 #include "sane/saneopts.h"
 #include "sane/sanei_scsi.h"
-#include "ricoh.h"
 
 #define BACKEND_NAME ricoh
 #include "sane/sanei_backend.h"
@@ -70,6 +69,8 @@
 #include "sane/sanei_config.h"
 #define RICOH_CONFIG_FILE "ricoh.conf"
 
+#include "ricoh.h"
+
 #define MM_PER_INCH	25.4
 #define MAX(a,b)	((a) > (b) ? (a) : (b))
 
@@ -142,15 +143,15 @@ attach (const char *devnam, Ricoh_Device ** devp)
     }
 
   if (ibuf.devtype != 6
-      || strncmp (ibuf.vendor, "RICOH", 5) != 0
-      || (strncmp (ibuf.product, "IS50", 4) != 0
-	  && strncmp (ibuf.product, "IS60", 4) != 0))
+      || strncmp ((char *)ibuf.vendor, "RICOH", 5) != 0
+      || (strncmp ((char *)ibuf.product, "IS50", 4) != 0
+	  && strncmp ((char *)ibuf.product, "IS60", 4) != 0))
     {
       DBG (1, "attach: device doesn't look like the Ricoh scanner I know\n");
       sanei_scsi_close (fd);
       return (SANE_STATUS_INVAL);
     }
-  is50 = (strncmp (ibuf.product, "IS50", 4) == 0);
+  is50 = (strncmp ((char *)ibuf.product, "IS50", 4) == 0);
 
   DBG (3, "attach: sending TEST_UNIT_READY\n");
   status = test_unit_ready (fd);
@@ -223,8 +224,8 @@ attach (const char *devnam, Ricoh_Device ** devp)
   dev->sane.vendor = "RICOH";
   str = malloc (16 + 1);
   memset (str, 0, sizeof (str));
-  strncpy (str, ibuf.product, sizeof(ibuf.product));
-  strncpy (str + sizeof(ibuf.revision), ibuf.revision, sizeof(ibuf.revision));
+  strncpy (str, (char *)ibuf.product, sizeof(ibuf.product));
+  strncpy (str + sizeof(ibuf.revision), (char *)ibuf.revision, sizeof(ibuf.revision));
   str[sizeof(ibuf.product) + sizeof(ibuf.revision)] = '\0';
   dev->sane.model = str;
   dev->sane.type = "flatbed scanner";
@@ -326,6 +327,7 @@ init_options (Ricoh_Scanner * s)
 
   s->opt[OPT_NUM_OPTS].title = SANE_TITLE_NUM_OPTIONS;
   s->opt[OPT_NUM_OPTS].desc = SANE_DESC_NUM_OPTIONS;
+  s->opt[OPT_NUM_OPTS].type = SANE_TYPE_INT;
   s->opt[OPT_NUM_OPTS].cap = SANE_CAP_SOFT_DETECT;
   s->val[OPT_NUM_OPTS].w = NUM_OPTIONS;
 
