@@ -47,7 +47,7 @@
    of the the AGFA SnapScan and Acer Vuego/Prisa flatbed scanners. */
 
 
-/* $Id: snapscan-sources.c,v 1.12 2004/11/14 19:26:38 oliver-guest Exp $
+/* $Id: snapscan-sources.c,v 1.21 2005-12-02 19:12:54 oliver-guest Exp $
    SnapScan backend data sources (implementation) */
 
 /**************************************************************************************
@@ -667,16 +667,16 @@ static SANE_Status create_Expander (SnapScan_Scanner *pss,
 /* 
    This filter implements a fix for scanners that have some columns
    of pixels offset. Currently it only shifts every other column
-   starting with the first one down 4 pixels.
+   starting with the first one down ch_offset pixels.
    
    The Deinterlacer detects if data is in SANE RGB frame format (3 bytes/pixel)
    or in Grayscale (1 byte/pixel).
    
-   The first 4 lines of data in the output are fudged so that even indexed
+   The first ch_offset lines of data in the output are fudged so that even indexed
    add odd indexed pixels will have the same value. This is necessary because
    the real pixel values of the columns that are shifted down are not 
-   in the data for the first 4 lines. A better way to handle this would be to
-   scan in 4 extra lines of data, but I haven't figured out how to do this 
+   in the data for the first ch_offset lines. A better way to handle this would be to
+   scan in ch_offset extra lines of data, but I haven't figured out how to do this 
    yet.
 
 */
@@ -686,10 +686,14 @@ typedef struct
     TX_SOURCE_GUTS;
     SANE_Byte *ch_buf;            /* channel buffer */
     SANE_Int   ch_size;           /* channel buffer size */
+    SANE_Int   ch_line_size;      /* size of one line */
     SANE_Int   ch_ndata;          /* actual #bytes in channel buffer */
     SANE_Int   ch_pos;            /* position in buffer */
     SANE_Int   ch_bytes_per_pixel;
+    SANE_Bool  ch_lineart;
+    SANE_Int   ch_offset;         /* The number of lines to be shifted */
     SANE_Bool  ch_past_init;      /* flag indicating if we have enough data to shift pixels down */
+    SANE_Bool  ch_shift_even;     /* flag indicating wether even or odd pixels are shifted */
 } Deinterlacer;
 
 static SANE_Int Deinterlacer_remaining (Source *pself)
@@ -716,17 +720,17 @@ static SANE_Status Deinterlacer_get (Source *pself, SANE_Byte *pbuf, SANE_Int *p
            pself->remaining(pself) > 0 &&
            !cancelRead)
     {
-        if (ps->ch_pos % (ps->ch_size/5) == ps->ch_ndata % (ps->ch_size/5) )
+        if (ps->ch_pos % (ps->ch_line_size) == ps->ch_ndata % (ps->ch_line_size) )
         {
             /* we need more data; try to get the remainder of the current
                line, or else the next line */
-            SANE_Int ndata = (ps->ch_size/5) - ps->ch_ndata % (ps->ch_size/5);
+            SANE_Int ndata = (ps->ch_line_size) - ps->ch_ndata % (ps->ch_line_size);
             if (ps->ch_pos >= ps->ch_size)
             {
 	        /* wrap to the beginning of the buffer if we need to */
                 ps->ch_ndata = 0;
                 ps->ch_pos = 0;
-                ndata = ps->ch_size /5;
+                ndata = ps->ch_line_size;
             }
             status = TxSource_get(pself, ps->ch_buf + ps->ch_pos, &ndata);
             if (status != SANE_STATUS_GOOD)
@@ -735,30 +739,70 @@ static SANE_Status Deinterlacer_get (Source *pself, SANE_Byte *pbuf, SANE_Int *p
                 break;
             ps->ch_ndata += ndata;
         }
-	
-        if ((ps->ch_pos/ps->ch_bytes_per_pixel) % 2 == 0)
+        /* Handle special lineart mode: Valid pixels need to be masked */
+        if (ps->ch_lineart)
         {
-	    /* the even indexed pixels need to be shifted down */
-	    if (ps->ch_past_init){
-	    	/* We need to use data 4 lines back */
-		/* So we just go one forward and it will wrap around to 4 back. */
-	    	*pbuf = ps->ch_buf[(ps->ch_pos + (ps->ch_size/5)) % ps->ch_size];
-	    }else{
-		/* Use data from the next pixel for even indexed pixels
-		   if we are on the first few lines. 
-	           TODO: also we will overread the buffer if the buffer read ended 
-		   on the first pixel. */
-		if (ps->ch_pos % (ps->ch_size/5) == 0 )
-	    	    *pbuf = ps->ch_buf[ps->ch_pos+ps->ch_bytes_per_pixel];
-		else
-	    	    *pbuf = ps->ch_buf[ps->ch_pos-ps->ch_bytes_per_pixel];
-	    }
-        }else{
-	    /* odd indexed pixels are okay */
-            *pbuf = ps->ch_buf[ps->ch_pos];
+            if (ps->ch_past_init)
+            {
+                if (ps->ch_shift_even)
+                {
+                    /* Even columns need to be shifted, i.e. bits 1,3,5,7 -> 0xaa */
+                    /* use valid pixels from this line and shifted pixels from ch_size lines back */
+                    *pbuf = (ps->ch_buf[ps->ch_pos] & 0x55) |
+                            (ps->ch_buf[(ps->ch_pos + (ps->ch_line_size)) % ps->ch_size] & 0xaa);
+                }
+                else
+                {
+                    /* Odd columns need to be shifted, i.e. bits 0,2,4,6 -> 0x55 */
+                    *pbuf = (ps->ch_buf[ps->ch_pos] & 0xaa) |
+                            (ps->ch_buf[(ps->ch_pos + (ps->ch_line_size)) % ps->ch_size] & 0x55);
+                }
+            }
+            else
+            {
+                /* not enough data. duplicate pixel values from previous column */
+                if (ps->ch_shift_even)
+                {
+                    /* bits 0,2,4,6 contain valid data -> 0x55 */
+                    SANE_Byte valid_pixel = ps->ch_buf[ps->ch_pos] & 0x55;
+                    *pbuf = valid_pixel | (valid_pixel >> 1);
+                }
+                else
+                {
+
+                    /* bits 1,3,5,7 contain valid data -> 0xaa */
+                    SANE_Byte valid_pixel = ps->ch_buf[ps->ch_pos] & 0xaa;
+                    *pbuf = valid_pixel | (valid_pixel << 1);
+                }
+            }
+        }
+        else /* colour / grayscale mode */
+        {
+            if ((ps->ch_shift_even && ((ps->ch_pos/ps->ch_bytes_per_pixel) % 2 == 0)) ||
+                (!ps->ch_shift_even && ((ps->ch_pos/ps->ch_bytes_per_pixel) % 2 == 1)))
+            {
+                /* the even indexed pixels need to be shifted down */
+                if (ps->ch_past_init){
+                    /* We need to use data 4 lines back */
+                    /* So we just go one forward and it will wrap around to 4 back. */
+                    *pbuf = ps->ch_buf[(ps->ch_pos + (ps->ch_line_size)) % ps->ch_size];
+                }else{
+                    /* Use data from the next pixel for even indexed pixels
+                      if we are on the first few lines. 
+                      TODO: also we will overread the buffer if the buffer read ended 
+                      on the first pixel. */
+                    if (ps->ch_pos % (ps->ch_line_size) == 0 )
+                        *pbuf = ps->ch_buf[ps->ch_pos+ps->ch_bytes_per_pixel];
+                    else
+                        *pbuf = ps->ch_buf[ps->ch_pos-ps->ch_bytes_per_pixel];
+                }
+            }else{
+                /* odd indexed pixels are okay */
+                *pbuf = ps->ch_buf[ps->ch_pos];
+            }
         }
 	/* set the flag so we know we have enough data to start shifting columns */
-	if (ps->ch_pos >= ps->ch_size /5 *4)
+	if (ps->ch_pos >= ps->ch_line_size * ps->ch_offset)
 	   ps->ch_past_init = SANE_TRUE;
 
         pbuf++;
@@ -786,6 +830,7 @@ static SANE_Status Deinterlacer_done (Source *pself)
     free(ps->ch_buf);
     ps->ch_buf = NULL;
     ps->ch_size = 0;
+    ps->ch_line_size = 0;
     ps->ch_pos = 0;
     return status;
 }
@@ -804,8 +849,23 @@ static SANE_Status Deinterlacer_init (Deinterlacer *pself,
                                        psub);
     if (status == SANE_STATUS_GOOD)
     {
-        /* We need at least 5 lines of buffer in order to shift up 4 pixels. */
-        pself->ch_size = TxSource_bytesPerLine((Source *) pself) *5;
+        pself->ch_shift_even = SANE_TRUE;
+        switch (pss->pdev->model)
+        {
+        case PERFECTION3490:
+            pself->ch_offset = 8;
+            if ((actual_mode(pss) == MD_GREYSCALE) || (actual_mode(pss) == MD_LINEART))
+                 pself->ch_shift_even = SANE_FALSE;
+            break;
+        case PERFECTION2480:
+        default:
+            pself->ch_offset = 4;
+            break;
+        }
+        pself->ch_line_size = TxSource_bytesPerLine((Source *) pself);
+        /* We need at least ch_offset+1 lines of buffer in order 
+           to shift up ch_offset pixels. */
+        pself->ch_size = pself->ch_line_size * (pself->ch_offset + 1);
         pself->ch_buf = (SANE_Byte *) malloc(pself->ch_size);
         if (pself->ch_buf == NULL)
         {
@@ -819,11 +879,14 @@ static SANE_Status Deinterlacer_init (Deinterlacer *pself,
             pself->ch_ndata = 0;
             pself->ch_pos = 0;
             pself->ch_past_init = SANE_FALSE;
-	    if (actual_mode(pss) == MD_GREYSCALE)
+	    if ((actual_mode(pss) == MD_GREYSCALE) || (actual_mode(pss) == MD_LINEART))
 	    	pself->ch_bytes_per_pixel = 1;
 	    else
 	    	pself->ch_bytes_per_pixel = 3;
+	    if (pss->bpp_scan == 16)
+		pself->ch_bytes_per_pixel *= 2;
         }
+        pself->ch_lineart = (actual_mode(pss) == MD_LINEART);
     }
     return status;
 }
@@ -929,9 +992,22 @@ static SANE_Status RGBRouter_get (Source *pself,
             b = (ps->cb_start + ps->ch_offset[2])%ps->cb_size;
             for (i = 0;  i < ps->cb_line_size/3;  i++)
             {
-                *s++ = ps->cbuf[r++];
-                *s++ = ps->cbuf[g++];
-                *s++ = ps->cbuf[b++];
+                if (pself->pss->bpp_scan == 8)
+                {
+                    *s++ = ps->cbuf[r++];
+                    *s++ = ps->cbuf[g++];
+                    *s++ = ps->cbuf[b++];
+                }
+                else
+                {
+                    *s++ = ps->cbuf[r++];
+                    *s++ = ps->cbuf[r++];
+                    *s++ = ps->cbuf[g++];
+                    *s++ = ps->cbuf[g++];
+                    *s++ = ps->cbuf[b++];
+                    *s++ = ps->cbuf[b++];
+                    i++;
+                }
             }
 
             /* end of reading & offsetiing whole line data;
@@ -1125,8 +1201,9 @@ static SANE_Status create_source_chain (SnapScan_Scanner *pss,
                some scanners like the Epson Perfection 2480/2580 
                at 2400 dpi. */
             if (status == SANE_STATUS_GOOD && 
-                pss->pdev->model == PERFECTION2480 &&
-                pss->res == 2400)
+                ((pss->pdev->model == PERFECTION2480 && pss->res == 2400) ||
+                (pss->pdev->model == PERFECTION3490 && pss->res == 3200) ||
+                (pss->pdev->model == PRISA5000E && pss->res == 1200)))
                 status = create_Deinterlacer (pss, *pps, pps);
             break;
         case MD_BILEVELCOLOUR:
@@ -1134,13 +1211,15 @@ static SANE_Status create_source_chain (SnapScan_Scanner *pss,
             if (status == SANE_STATUS_GOOD)
                 status = create_RGBRouter (pss, *pps, pps);
             if (status == SANE_STATUS_GOOD && 
-                pss->pdev->model == PERFECTION2480 &&
-                pss->res == 2400)
+                ((pss->pdev->model == PERFECTION2480 && pss->res == 2400) ||
+                (pss->pdev->model == PERFECTION3490 && pss->res == 3200) ||
+                (pss->pdev->model == PRISA5000E && pss->res == 1200)))
                 status = create_Deinterlacer (pss, *pps, pps);
             break;
         case MD_GREYSCALE:
-            if (pss->pdev->model == PERFECTION2480 &&
-                pss->res == 2400)
+            if ((pss->pdev->model == PERFECTION2480 && pss->res == 2400) ||
+                (pss->pdev->model == PERFECTION3490 && pss->res == 3200) ||
+                (pss->pdev->model == PRISA5000E && pss->res == 1200))
                 status = create_Deinterlacer (pss, *pps, pps);
             break;
         case MD_LINEART:
@@ -1149,6 +1228,8 @@ static SANE_Status create_source_chain (SnapScan_Scanner *pss,
                the internal meaning of "negative" is reversed */
             if (pss->negative == SANE_FALSE)
                 status = create_Inverter (pss, *pps, pps);
+            if (pss->pdev->model == PERFECTION3490 && pss->res == 3200)
+                status = create_Deinterlacer (pss, *pps, pps);
             break;
         default:
             DBG (DL_MAJOR_ERROR, "%s: bad mode value %d (internal error)\n",
@@ -1162,6 +1243,33 @@ static SANE_Status create_source_chain (SnapScan_Scanner *pss,
 
 /*
  * $Log: snapscan-sources.c,v $
+ * Revision 1.21  2005-12-02 19:12:54  oliver-guest
+ * Another fix for lineart mode for the Epson 3490 @ 3200 DPI
+ *
+ * Revision 1.20  2005/11/28 19:28:29  oliver-guest
+ * Fix for lineart mode of Epson 3490 @ 3200 DPI
+ *
+ * Revision 1.19  2005/11/25 17:24:48  oliver-guest
+ * Fix for Epson 3490 @ 3200 DPI for grayscale and lineart mode
+ *
+ * Revision 1.18  2005/11/17 23:47:11  oliver-guest
+ * Revert previous 'fix', disable 2400 dpi for Epson 3490, use 1600 dpi instead
+ *
+ * Revision 1.17  2005/11/17 23:32:23  oliver-guest
+ * Fixes for Epson 3490 @ 2400 DPI
+ *
+ * Revision 1.16  2005/11/10 19:42:02  oliver-guest
+ * Added deinterlacing for Epson 3490
+ *
+ * Revision 1.15  2005/10/31 21:08:47  oliver-guest
+ * Distinguish between Benq 5000/5000E/5000U
+ *
+ * Revision 1.14  2005/10/13 22:43:30  oliver-guest
+ * Fixes for 16 bit scan mode from Simon Munton
+ *
+ * Revision 1.13  2005/10/11 18:47:07  oliver-guest
+ * Fixes for Epson 3490 and 16 bit scan mode
+ *
  * Revision 1.12  2004/11/14 19:26:38  oliver-guest
  * Applied patch from Julien Blache to change ch_past_init from SANE_Int to SANE_Bool
  *
