@@ -1,7 +1,7 @@
 #ifndef SP15C_SCSI_H
 #define SP15C_SCSI_H
 
-static const char RCSid_sh[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c-scsi.h,v 1.6 2005/09/19 19:57:48 fzago-guest Exp $";
+static const char RCSid_sh[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c-scsi.h,v 1.6 2005-09-19 19:57:48 fzago-guest Exp $";
 /* sane - Scanner Access Now Easy.
 
    This file is part of the SANE package.
@@ -49,7 +49,7 @@ static const char RCSid_sh[] = "$Header: /cvsroot/sane/sane-backends/backend/sp1
 /* ------------------------------------------------------------------------- */
 /*
  * $Log: sp15c-scsi.h,v $
- * Revision 1.6  2005/09/19 19:57:48  fzago-guest
+ * Revision 1.6  2005-09-19 19:57:48  fzago-guest
  * Replaced __unused__ with __sane_unused__ to avoid a namespace conflict.
  *
  * Revision 1.5  2004/11/13 19:53:04  fzago-guest
