@@ -1,4 +1,4 @@
-static const char RCSid[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c.c,v 1.13 2006/03/29 20:48:50 hmg-guest Exp $";
+static const char RCSid[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c.c,v 1.15 2007-11-18 10:59:18 ellert-guest Exp $";
 /* sane - Scanner Access Now Easy.
 
    This file is part of the SANE package.
@@ -45,6 +45,12 @@ static const char RCSid[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c.
 
 /*
  * $Log: sp15c.c,v $
+ * Revision 1.15  2007-11-18 10:59:18  ellert-guest
+ * Fix handling of valid "negative" PIDs
+ *
+ * Revision 1.14  2007-10-26 14:56:38  jblache
+ * OPT_NUM_OPTS must be of type SANE_TYPE_INT.
+ *
  * Revision 1.13  2006/03/29 20:48:50  hmg-guest
  * Fixed ADF support. Patch from Andreas Degert <ad@papyrus-gmbh.de>.
  *
@@ -1385,6 +1391,7 @@ init_options (struct sp15c *scanner)
 
   scanner->opt[OPT_NUM_OPTS].title = SANE_TITLE_NUM_OPTIONS;
   scanner->opt[OPT_NUM_OPTS].desc = SANE_DESC_NUM_OPTIONS;
+  scanner->opt[OPT_NUM_OPTS].type = SANE_TYPE_INT;
   scanner->opt[OPT_NUM_OPTS].cap = SANE_CAP_SOFT_DETECT;
 
 /************** "Mode" group: **************/
@@ -1743,7 +1750,7 @@ do_cancel (struct sp15c *scanner)
 
   do_eof (scanner);             /* close pipe and reposition scanner */
 
-  if (scanner->reader_pid > 0)
+  if (scanner->reader_pid != -1)
     {
       int exit_status;
       DBG (10, "do_cancel: kill reader_process\n");
@@ -1751,7 +1758,7 @@ do_cancel (struct sp15c *scanner)
       sanei_thread_kill (scanner->reader_pid);
       DBG (50, "wait for scanner to stop\n");
       sanei_thread_waitpid (scanner->reader_pid, &exit_status);
-      scanner->reader_pid = 0;
+      scanner->reader_pid = -1;
     }
 
   if (scanner->sfd >= 0)
