@@ -1,7 +1,7 @@
 /* sane - Scanner Access Now Easy.
 
    Copyright (C) 2002-2003 Frank Zago (sane at zago dot net)
-   Copyright (C) 2003-2005 Gerard Klaver (gerard at gkall dot hobby dot nl)
+   Copyright (C) 2003-2008 Gerard Klaver (gerard at gkall dot hobby dot nl)
 
    This file is part of the SANE package.
    
@@ -43,7 +43,7 @@
 */
 
 /*
-   $Id: teco2.c,v 1.16 2005/07/29 20:27:12 gekl-guest Exp $
+   $Id: teco2.c,v 1.19 2008-01-12 14:25:50 gekl-guest Exp $
    TECO scanner VM3575, VM656A, VM6575, VM6586, VM356A, VM3564
    update 2003/02/14, Patch for VM356A Gerard Klaver
    update 2003/03/19, traces, tests VM356A Gerard Klaver, Michael Hoeller
@@ -59,13 +59,15 @@
 		      VM656A, VM6586
    update 2004/08/05, use of SANE_VALUE_SCAN_MODE_LINEART, _GRAY, and _COLOR,
                       changed use of %d to %ld (when bytes values are displayed)
-   update 2005/03/04, use of __unused__
+   update 2005/03/04, use of __sane_unused__
    update 2005/07/29. Removed using teco_request_sense (dev) routine for VM3564
+   update 2008/01/12, Update teco_request_sense routine due to no 
+                      init value for size.
 */
 
 /*--------------------------------------------------------------------------*/
 
-#define BUILD 9			/* 2005/07/29 */
+#define BUILD 10			/* 2008/01/12 */
 #define BACKEND_NAME teco2
 #define TECO2_CONFIG_FILE "teco2.conf"
 
@@ -1518,29 +1520,38 @@ teco_do_calibration (Teco_Scanner * dev)
   return SANE_STATUS_GOOD;
 }
 
-      /*------------request sense command 03----------------*/
+/*------------request sense command 03----------------*/
 static SANE_Status
 teco_request_sense (Teco_Scanner * dev)
 {
-  CDB cdb;
   SANE_Status status;
+  unsigned char buf[255];
+  CDB cdb;
   size_t size;
+  /* size = 0;  */
 
-
+  DBG (DBG_proc, "teco_request_sense: enter\n");
+  
+  size = sizeof (buf);
   MKSCSI_REQUEST_SENSE (cdb, size);
-  size = cdb.data[5];
+  
+  /*size = cdb.data[5];
 
-  hexdump (DBG_info2, "teco_request_sense: enter\n", cdb.data, cdb.len);
+  hexdump (DBG_info2, "teco_request_sense", cdb.data, cdb.len);
 
   status = sanei_scsi_cmd2 (dev->sfd, cdb.data, cdb.len,
-			    NULL, 0, dev->buffer, &size);
+			    NULL, 0, dev->buffer, &size); */
+  status = sanei_scsi_cmd2 (dev->sfd, cdb.data, cdb.len,
+			    NULL, 0, buf, &size);
 
+  hexdump (DBG_info2, "teco_request_sense:", buf, size);
+  
   DBG (DBG_proc, "teco_request_sense: exit, status=%d\n", status);
 
   return (status);
 }
 
-/*----------------------------------------------------------*/
+/*----------------------------------------------------*/
 
 /* Send the gamma */
 static SANE_Status
@@ -1669,7 +1680,7 @@ teco_scan (Teco_Scanner * dev)
 
 /* SCSI sense handler. Callback for SANE. */
 static SANE_Status
-teco_sense_handler (int scsi_fd, unsigned char *result, void __unused__ *arg)
+teco_sense_handler (int scsi_fd, unsigned char *result, void __sane_unused__ *arg)
 {
   int asc, ascq, sensekey;
   int len;
@@ -2504,7 +2515,7 @@ do_cancel (Teco_Scanner * dev)
 /* Sane entry points */
 
 SANE_Status
-sane_init (SANE_Int * version_code, SANE_Auth_Callback __unused__ authorize)
+sane_init (SANE_Int * version_code, SANE_Auth_Callback __sane_unused__ authorize)
 {
   FILE *fp;
   char dev_name[PATH_MAX];
@@ -2516,7 +2527,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback __unused__ authorize)
 
   DBG (DBG_error, "This is sane-teco2 version %d.%d-%d\n", V_MAJOR,
        V_MINOR, BUILD);
-  DBG (DBG_error, "(C) 2002 - 2003 by Frank Zago, update 2003 - 2005 by Gerard Klaver\n");
+  DBG (DBG_error, "(C) 2002 - 2003 by Frank Zago, update 2003 - 2008 by Gerard Klaver\n");
 
   if (version_code)
     {
@@ -2551,7 +2562,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback __unused__ authorize)
 }
 
 SANE_Status
-sane_get_devices (const SANE_Device *** device_list, SANE_Bool __unused__ local_only)
+sane_get_devices (const SANE_Device *** device_list, SANE_Bool __sane_unused__ local_only)
 {
   Teco_Scanner *dev;
   int i;
@@ -3341,7 +3352,7 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
 }
 
 SANE_Status
-sane_set_io_mode (SANE_Handle __unused__ handle, SANE_Bool __unused__ non_blocking)
+sane_set_io_mode (SANE_Handle __sane_unused__ handle, SANE_Bool __sane_unused__ non_blocking)
 {
   SANE_Status status;
   Teco_Scanner *dev = handle;
@@ -3368,7 +3379,7 @@ sane_set_io_mode (SANE_Handle __unused__ handle, SANE_Bool __unused__ non_blocki
 }
 
 SANE_Status
-sane_get_select_fd (SANE_Handle __unused__ handle, SANE_Int __unused__ * fd)
+sane_get_select_fd (SANE_Handle __sane_unused__ handle, SANE_Int __sane_unused__ * fd)
 {
   DBG (DBG_proc, "sane_get_select_fd: enter\n");
 
