@@ -80,7 +80,7 @@
 #include "../include/sane/sanei.h"
 #include "../include/sane/saneopts.h"
 
-#define BACKEND_VERSION "0.02-8"
+#define BACKEND_VERSION "0.02-11"
 #define BACKEND_NAME    u12
 #include "../include/sane/sanei_backend.h"
 #include "../include/sane/sanei_config.h"
@@ -143,9 +143,9 @@ static ModeParam mode_params[] =
 
 static const SANE_String_Const mode_list[] =
 {
-	SANE_I18N("Binary"),
-	SANE_I18N("Gray"),
-	SANE_I18N("Color"),
+	SANE_VALUE_SCAN_MODE_LINEART,
+	SANE_VALUE_SCAN_MODE_GRAY,
+	SANE_VALUE_SCAN_MODE_COLOR,
 	SANE_I18N("Color 36"),
 	NULL
 };
@@ -320,6 +320,9 @@ static int reader_process( void *args )
 
 	sigfillset ( &ignore_set );
 	sigdelset  ( &ignore_set, SIGTERM );
+#if defined (__APPLE__) && defined (__MACH__)
+	sigdelset  ( &ignore_set, SIGUSR2 );
+#endif
 	sigprocmask( SIG_SETMASK, &ignore_set, 0 );
 
 	cancelRead = SANE_FALSE;
@@ -388,7 +391,7 @@ static SANE_Status do_cancel( U12_Scanner *scanner, SANE_Bool closepipe )
 
 	scanner->scanning = SANE_FALSE;
 
-	if( scanner->reader_pid > 0 ) {
+	if( scanner->reader_pid != -1 ) {
 
 		DBG( _DBG_PROC, ">>>>>>>> killing reader_process <<<<<<<<\n" );
 
@@ -418,7 +421,7 @@ static SANE_Status do_cancel( U12_Scanner *scanner, SANE_Bool closepipe )
 			sanei_thread_sendsig( scanner->reader_pid, SIGKILL );
 #endif
 		}
-		scanner->reader_pid = 0;
+		scanner->reader_pid = -1;
 		DBG( _DBG_PROC, "reader_process killed\n");
 
 		if( scanner->hw->fd >= 0 ) {
@@ -845,8 +848,8 @@ static SANE_Status attach( const char *dev_name,
 	dev->sane.name   = dev->name;           /* compiler warnings        */
 	dev->sane.vendor = "Plustek";
 	dev->sane.model  = "U12/1212U";
-	dev->sane.type   = "USB flatbed scanner";
-	dev->initialized = SANE_FALSE;         
+	dev->sane.type   = SANE_I18N ("flatbed scanner");
+	dev->initialized = SANE_FALSE;
 
 	memcpy( &dev->adj, &cnf->adj, sizeof(AdjDef));
 	show_cnf( cnf );
@@ -1728,9 +1731,9 @@ SANE_Status sane_start( SANE_Handle handle )
 	
 	cancelRead = SANE_FALSE;
 	
-	if( s->reader_pid < 0 ) {
+	if( s->reader_pid == -1 ) {
 		DBG( _DBG_ERROR, "ERROR: could not start reader task\n" );
-	    s->scanning = SANE_FALSE;
+		s->scanning = SANE_FALSE;
 		u12if_close( dev );
 		return SANE_STATUS_IO_ERROR;
 	}
