@@ -1,8 +1,12 @@
 /* scanimage -- command line scanning utility
    Uses the SANE library.
    Copyright (C) 1996, 1997, 1998 Andreas Beck and David Mosberger
+   
+   Copyright (C) 1999 - 2005 by the SANE Project -- See AUTHORS and ChangeLog
+   for details.
 
-   You can contact me at becka@uni-duesseldorf.de
+   For questions and comments contact the sane-devel mailinglist (see
+   http://www.sane-project.org/mailing-lists.html).
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -80,6 +84,7 @@ static struct option basic_options[] = {
   {"formatted-device-list", required_argument, NULL, 'f'},
   {"help", no_argument, NULL, 'h'},
   {"verbose", no_argument, NULL, 'v'},
+  {"progress", no_argument, NULL, 'p'},
   {"test", no_argument, NULL, 'T'},
   {"version", no_argument, NULL, 'V'},
   {"buffer_size", no_argument, NULL, 'B'},
@@ -99,7 +104,7 @@ static struct option basic_options[] = {
 #define OUTPUT_PNM      0
 #define OUTPUT_TIFF     1
 
-#define BASE_OPTSTRING	"d:hi:Lf:nvBVTb"
+#define BASE_OPTSTRING	"d:hi:Lf:nvBVTbp"
 #define STRIP_HEIGHT	256	/* # lines we increment image height */
 
 static struct option *all_options;
@@ -107,6 +112,7 @@ static int option_number_len;
 static int *option_number;
 static SANE_Handle device;
 static int verbose;
+static int progress = 0;
 static int test;
 static int output_format = OUTPUT_PNM;
 static int help;
@@ -172,7 +178,7 @@ auth_callback (SANE_String_Const resource,
 	      else
 		len = strlen (resource);
 
-	      while (fgets (tmp, 512, pass_file))
+	      while (fgets (tmp, sizeof(tmp), pass_file))
 		{
 
 		  if ((strlen (tmp) > 0) && (tmp[strlen (tmp) - 1] == '\n'))
@@ -709,7 +715,14 @@ parse_scalar (const SANE_Option_Descriptor * opt, const char *str,
 	str += sizeof ("us") - 1;
       break;
     }
-  *value = v + 0.5;
+
+  if(v < 0){
+    *value = v - 0.5;
+  }
+  else{
+    *value = v + 0.5;
+  }
+
   return str;
 }
 
@@ -929,7 +942,14 @@ set_option (SANE_Handle device, int optnum, void *valuep)
   SANE_Int info;
 
   opt = sane_get_option_descriptor (device, optnum);
-
+  if (opt && (!SANE_OPTION_IS_ACTIVE (opt->cap)))
+    {
+      if (verbose > 0)
+	fprintf (stderr, "%s: ignored request to set inactive option %s\n",
+		 prog_name, opt->name);
+      return;
+    }
+    
   if (opt->size == sizeof (SANE_Word) && opt->type != SANE_TYPE_STRING)
     orig = *(SANE_Word *) valuep;
 
@@ -1115,7 +1135,7 @@ advance (Image * image)
 static SANE_Status
 scan_it (void)
 {
-  int i, len, first_frame = 1, offset = 0, must_buffer = 0;
+  int i, len, first_frame = 1, offset = 0, must_buffer = 0, hundred_percent;
   SANE_Byte min = 0xff, max = 0;
   SANE_Parameters parm;
   SANE_Status status;
@@ -1130,12 +1150,15 @@ scan_it (void)
 
   do
     {
-      status = sane_start (device);
-      if (status != SANE_STATUS_GOOD)
+      if (!first_frame)
 	{
-	  fprintf (stderr, "%s: sane_start: %s\n",
-		   prog_name, sane_strstatus (status));
-	  goto cleanup;
+	  status = sane_start (device);
+	  if (status != SANE_STATUS_GOOD)
+	    {
+	      fprintf (stderr, "%s: sane_start: %s\n",
+		       prog_name, sane_strstatus (status));
+	      goto cleanup;
+	    }
 	}
 
       status = sane_get_parameters (device, &parm);
@@ -1161,8 +1184,9 @@ scan_it (void)
 			 prog_name, parm.pixels_per_line,
 			 8 * parm.bytes_per_line / parm.pixels_per_line);
 	    }
+
 	  fprintf (stderr, "%s: acquiring %s frame\n", prog_name,
-		   format_name[parm.format]);
+	   parm.format <= SANE_FRAME_BLUE ? format_name[parm.format]:"Unknown");
 	}
 
       if (first_frame)
@@ -1199,6 +1223,9 @@ scan_it (void)
 				      parm.lines, parm.depth);
 		}
 	      break;
+
+            default:
+	      break;
 	    }
 
 	  if (must_buffer)
@@ -1237,11 +1264,20 @@ scan_it (void)
 	  offset = parm.format - SANE_FRAME_RED;
 	  image.x = image.y = 0;
 	}
+      hundred_percent = parm.bytes_per_line * parm.lines 
+	* ((parm.format == SANE_FRAME_RGB || parm.format == SANE_FRAME_GRAY) ? 1:3);
 
       while (1)
 	{
+	  double progr;
 	  status = sane_read (device, buffer, buffer_size, &len);
 	  total_bytes += (SANE_Word) len;
+          progr = ((total_bytes * 100.) / (double) hundred_percent);
+          if (progr > 100.)
+	    progr = 100.;
+          if (progress)
+	    fprintf (stderr, "Progress: %3.1f%%\r", progr);
+
 	  if (status != SANE_STATUS_GOOD)
 	    {
 	      if (verbose && parm.depth == 8)
@@ -1255,6 +1291,7 @@ scan_it (void)
 		}
 	      break;
 	    }
+
 	  if (must_buffer)
 	    {
 	      switch (parm.format)
@@ -1301,6 +1338,9 @@ scan_it (void)
 		    }
 		  offset += len;
 		  break;
+
+                default:
+	          break;
 		}
 	    }
 	  else			/* ! must_buffer */
@@ -1383,8 +1423,10 @@ scan_it (void)
 	}
     }
 
+  /* flush the output buffer */
+  fflush( stdout );
+
 cleanup:
-  sane_cancel (device);
   if (image.data)
     free (image.data);
 
@@ -1462,8 +1504,9 @@ test_it (void)
 	     "variable height at %d bits/pixel\n",
 	     prog_name, parm.pixels_per_line,
 	     8 * parm.bytes_per_line / parm.pixels_per_line);
-  fprintf (stderr, "%s: acquiring %s frame, %d bits/sample\n",
-	   prog_name, format_name[parm.format], parm.depth);
+  fprintf (stderr, "%s: acquiring %s frame, %d bits/sample\n", prog_name,
+	   parm.format <= SANE_FRAME_BLUE ? format_name[parm.format]:"Unknown",
+           parm.depth);
 
   image.data = malloc (parm.bytes_per_line * 2);
 
@@ -1621,6 +1664,9 @@ main (int argc, char **argv)
 	case 'v':
 	  ++verbose;
 	  break;
+	case 'p':
+          progress = 1;
+	  break;
 	case 'B':
 	  buffer_size = (1024 * 1024);
 	  break;
@@ -1792,6 +1838,8 @@ main (int argc, char **argv)
 Start image acquisition on a scanner device and write PNM image data to\n\
 standard output.\n\
 \n\
+Parameters are separated by a blank from single-character options (e.g.\n\
+-d epson) and by a \"=\" from multi-character options (e.g. --device-name=epson).\n\
 -d, --device-name=DEVICE   use a given scanner device (e.g. hp:/dev/scanner)\n\
     --format=pnm|tiff      file format of output file\n\
 -i, --icc-profile=PROFILE  include this ICC profile into TIFF file\n", prog_name);
@@ -1811,6 +1859,7 @@ standard output.\n\
     --batch-prompt         ask for pressing a key before scanning a page\n\
     --accept-md5-only      only accept authorization requests using md5\n");
       printf ("\
+-p, --progress             print progress messages\n\
 -n, --dont-scan            only set options, don't actually scan\n\
 -T, --test                 test backend thoroughly\n\
 -h, --help                 display this help message and exit\n\
@@ -1935,6 +1984,7 @@ standard output.\n\
 
 	    case 'd':
 	    case 'h':
+	    case 'p':
 	    case 'v':
 	    case 'V':
 	    case 'T':
@@ -2087,11 +2137,6 @@ List of available devices:", prog_name);
 	  if (batch)		/* format is NULL unless batch mode */
 	    sprintf (path, format, n);	/* love --(C++) */
 
-	  if (batch && NULL == freopen (path, "w", stdout))
-	    {
-	      fprintf (stderr, "cannot open %s\n", path);
-	      return SANE_STATUS_ACCESS_DENIED;
-	    }
 
 	  if (batch)
 	    {
@@ -2108,12 +2153,28 @@ List of available devices:", prog_name);
 		      fprintf (stderr, "Batch terminated, %d pages scanned\n",
 			       (n - batch_increment));
 		      fclose (stdout);
-		      unlink (path);
 		      break;	/* get out of this loop */
 		    }
 		}
 	      fprintf (stderr, "Scanning page %d\n", n);
 	    }
+
+	  status = sane_start (device);
+	  if (status != SANE_STATUS_GOOD)
+	    {
+	      fprintf (stderr, "%s: sane_start: %s\n",
+		       prog_name, sane_strstatus (status));
+	      fclose (stdout);
+	      break;
+	    }
+
+	  if (batch && NULL == freopen (path, "w", stdout))
+	    {
+	      fprintf (stderr, "cannot open %s\n", path);
+	      sane_cancel (device);
+	      return SANE_STATUS_ACCESS_DENIED;
+	    }
+
 	  status = scan_it ();
 	  if (batch)
 	    {
@@ -2141,6 +2202,8 @@ List of available devices:", prog_name);
       while ((batch
 	      && (batch_count == BATCH_COUNT_UNLIMITED || --batch_count))
 	     && SANE_STATUS_GOOD == status);
+
+      sane_cancel (device);
     }
   else
     status = test_it ();
