@@ -23,7 +23,7 @@
    MA 02111-1307, USA.
 */
 
-#define SANE_DESC_VERSION "3.2"
+#define SANE_DESC_VERSION "3.3"
 
 #define MAN_PAGE_LINK "http://www.sane-project.org/man/%s.5.html"
 #define COLOR_MINIMAL      "\"#B00000\""
@@ -76,7 +76,8 @@ typedef enum output_mode
   output_mode_usermap,
   output_mode_db,
   output_mode_udev,
-  output_mode_plist
+  output_mode_plist,
+  output_mode_hal
 }
 output_mode;
 
@@ -292,7 +293,7 @@ print_usage (char *program_name)
 	  "(multiple directories can be concatenated by \":\")\n");
   printf ("  -m|--mode mode         "
 	  "Output mode (ascii, html-backends-split, html-mfgs,\n"
-	  "                         xml, statistics, usermap, db, udev, plist)\n");
+	  "                         xml, statistics, usermap, db, udev, plist, hal)\n");
   printf ("  -t|--title \"title\"     The title used for HTML pages\n");
   printf ("  -i|--intro \"intro\"     A short description of the "
 	  "contents of the page\n");
@@ -394,6 +395,11 @@ get_options (int argc, char **argv)
 	      DBG_INFO ("Output mode: %s\n", optarg);
 	      mode = output_mode_plist;
 	    }
+	  else if (strcmp (optarg, "hal") == 0)
+	    {
+	      DBG_INFO ("Output mode: %s\n", optarg);
+	      mode = output_mode_hal;
+	    }
 	  else
 	    {
 	      DBG_ERR ("Unknown output mode: %s\n", optarg);
@@ -3100,7 +3106,7 @@ print_db_header (void)
      "# respectively):\n");
   printf 
     ("#\n"
-     "# 0xVVVV<tab>0xPPPP<tab>root:scanner<tab>0660<tab>[/usr/local/bin/foo.sh]\n"
+     "# 0xVVVV<tab>0xPPPP<tab>root:scanner<tab>0664<tab>[/usr/local/bin/foo.sh]\n"
      "# Fields:\n"
      "#   vendor ID\n"
      "#   product ID\n"
@@ -3134,7 +3140,7 @@ print_db (void)
 	  name = name->next;
 	}
       printf ("\n");
-      printf ("%s\t%s\troot:scanner\t0660\t\n", usbid->usb_vendor_id,
+      printf ("%s\t%s\troot:scanner\t0664\t\n", usbid->usb_vendor_id,
 	      usbid->usb_product_id);
       usbid = usbid->next;
     }
@@ -3153,44 +3159,81 @@ print_udev_header (void)
     ("#\n"
      "# udev rules file for supported USB devices\n"
      "#\n"
-     "# To add a USB device, add a rule to the list below between the SUBSYSTEM...\n"
-     "# and LABEL... lines.\n"
+     "# To add a USB device, add a rule to the list below between the\n"
+     "# LABEL=\"libsane_rules_begin\" and LABEL=\"libsane_rules_end\" lines.\n"
      "#\n"
-     "# To run a script when your device is plugged in, add RUN=\"/path/to/script\"\n"
+     "# To run a script when your device is plugged in, add RUN+=\"/path/to/script\"\n"
      "# to the appropriate rule.\n");
   printf
     ("#\n"
      "# The following list already contains a lot of scanners. If your scanner\n"
      "# isn't mentioned there, add it as explained above and mail the entry to\n"
      "# the sane-devel mailing list (sane-devel@lists.alioth.debian.org).\n"
-     "#\n");
+     "#\n\n");
 }
 
 static void
 print_udev (void)
 {
   usbid_type *usbid = create_usbids_table ();
+  int i;
 
   print_udev_header ();
-  printf ("\nACTION!=\"add\", GOTO=\"libsane_rules_end\"\n");
-  printf ("SUBSYSTEM!=\"usb_device\", GOTO=\"libsane_rules_end\"\n\n");
+  printf("ACTION!=\"add\", GOTO=\"libsane_rules_end\"\n"
+	 "ENV{DEVTYPE}==\"usb_device\", GOTO=\"libsane_create_usb_dev\"\n"
+	 "SUBSYSTEM==\"usb_device\", GOTO=\"libsane_rules_begin\"\n"
+	 "SUBSYSTEM!=\"usb_device\", GOTO=\"libsane_rules_end\"\n"
+	 "\n");
+
+  printf("# Kernel >= 2.6.22 jumps here\n"
+	 "LABEL=\"libsane_create_usb_dev\"\n"
+	 "\n");
+
+  printf("# For Linux >= 2.6.22 without CONFIG_USB_DEVICE_CLASS=y\n"
+	 "# If the following rule does not exist on your system yet, uncomment it\n"
+	 "# ENV{DEVTYPE}==\"usb_device\", NAME=\"bus/usb/$env{BUSNUM}/$env{DEVNUM}\", "
+	 "MODE=\"0664\", OWNER=\"root\", GROUP=\"root\"\n"
+	 "\n");
+
+  printf("# Kernel < 2.6.22 jumps here\n"
+	 "LABEL=\"libsane_rules_begin\"\n"
+	 "\n");
+
   while (usbid)
     {
       manufacturer_model_type * name = usbid->name;
 
+      i = 0;
       printf ("# ");
       while (name)
 	{
-	  if (name != usbid->name)
+	  if ((name != usbid->name) && (i > 0))
 	    printf (" | ");
 	  printf ("%s", name->name);
 	  name = name->next;
+
+	  i++;
+
+	  /*
+	   * Limit the number of model names on the same line to 3,
+	   * as udev cannot handle very long lines and prints a warning
+	   * message while loading the rules files.
+	   */
+	  if ((i == 3) && (name != NULL))
+	    {
+	      printf("\n# ");
+	      i = 0;
+	    }
 	}
       printf ("\n");
-      printf ("SYSFS{idVendor}==\"%s\", SYSFS{idProduct}==\"%s\", MODE=\"660\", GROUP=\"scanner\"\n",
+      printf ("SYSFS{idVendor}==\"%s\", SYSFS{idProduct}==\"%s\", MODE=\"0664\", GROUP=\"scanner\", ENV{libsane_matched}=\"yes\"\n",
 	      usbid->usb_vendor_id + 2,  usbid->usb_product_id + 2);
       usbid = usbid->next;
     }
+
+  printf("\n# The following rule will disable USB autosuspend for the device\n");
+  printf("ENV{libsane_matched}==\"yes\", RUN+=\"/bin/sh -c 'test -e /sys/$env{DEVPATH}/power/level && echo on > /sys/$env{DEVPATH}/power/level'\"\n");
+
   printf ("\nLABEL=\"libsane_rules_end\"\n");
 }
 
@@ -3227,6 +3270,46 @@ print_plist (void)
   printf ("</plist>\n");
 }
 
+
+static void
+print_hal (void)
+{
+  int i;
+  usbid_type *usbid = create_usbids_table ();
+  printf ("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
+  printf ("<deviceinfo version=\"0.2\">\n");
+  printf ("  <device>\n");
+  printf ("    <match key=\"info.bus\" string=\"usb\">\n");
+  while (usbid)
+    {
+      manufacturer_model_type * name = usbid->name;
+
+      i = 0;
+      printf ("      <!-- ");
+      while (name)
+	{
+	  if ((name != usbid->name) && (i > 0))
+	    printf (" | ");
+	  printf ("%s", name->name);
+	  name = name->next;
+	  i++;
+	}
+      printf (" -->\n");
+      printf ("      <match key=\"usb.vendor_id\" int=\"%s\">\n", usbid->usb_vendor_id);
+      printf ("        <match key=\"usb.product_id\" int=\"%s\">\n", usbid->usb_product_id);
+      printf ("          <append key=\"info.capabilities\" type=\"strlist\">scanner</append>\n");
+      printf ("          <merge key=\"scanner.access_method\" type=\"string\">proprietary</merge>\n");
+      printf ("        </match>\n");
+      printf ("      </match>\n");
+      usbid = usbid->next;
+    }
+
+
+  printf ("    </match>\n");
+  printf ("  </device>\n");
+  printf ("</deviceinfo>\n");
+}
+
 int
 main (int argc, char **argv)
 {
@@ -3269,6 +3352,9 @@ main (int argc, char **argv)
     case output_mode_plist:
       print_plist ();
       break;
+    case output_mode_hal:
+      print_hal ();
+      break;
     default:
       DBG_ERR ("Unknown output mode\n");
       return 1;
