@@ -38,7 +38,6 @@
 
 #include "../include/sane/sanei.h"
 #include "../include/sane/sanei_scsi.h"
-#include "../include/sane/sanei_usb.h"
 #include "../include/sane/sanei_pa4s2.h"
 #include "../include/sane/sanei_config.h"
 
@@ -47,6 +46,8 @@
 extern char * check_usb_chip (struct usb_device *dev, int verbosity, SANE_Bool from_file);
 #endif
 
+#include "../include/sane/sanei_usb.h"
+
 #ifndef PATH_MAX
 # define PATH_MAX 1024
 #endif
@@ -721,7 +722,7 @@ get_next_file (char *dir_name, DIR * dir)
 
 #if defined (HAVE_WINDOWS_H)
 /* Return a list of potential scanners. There's a lot of hardcoded values here that might break on a system with lots of scsi devices. */
-static char **build_scsi_dev_list()
+static char **build_scsi_dev_list(void)
 {
 	char **dev_list;
 	int dev_list_index;
@@ -819,6 +820,7 @@ static char **build_scsi_dev_list()
 #endif
 
 #if defined (HAVE_IOKIT_CDB_IOSCSILIB_H) || \
+    defined (HAVE_IOKIT_SCSI_SCSICOMMANDOPERATIONCODES_H) || \
     defined (HAVE_IOKIT_SCSI_COMMANDS_SCSICOMMANDOPERATIONCODES_H)
 char **scsi_dev_list;
 int scsi_dev_list_index;
@@ -833,7 +835,7 @@ static SANE_Status AddToSCSIDeviceList (const char *dev) {
     return SANE_STATUS_NO_MEM;
 }
 
-static char **build_scsi_dev_list()
+static char **build_scsi_dev_list(void)
 {
   scsi_dev_list_index = 0;
   scsi_dev_list = malloc (100 * sizeof(char *));
@@ -1410,6 +1412,7 @@ main (int argc, char **argv)
 
 #if defined (HAVE_WINDOWS_H) || \
     defined (HAVE_IOKIT_CDB_IOSCSILIB_H) || \
+    defined (HAVE_IOKIT_SCSI_SCSICOMMANDOPERATIONCODES_H) || \
     defined (HAVE_IOKIT_SCSI_COMMANDS_SCSICOMMANDOPERATIONCODES_H)
    /* Build a list of valid of possible scanners found */
       dev_list = build_scsi_dev_list();
